@@ -19,12 +19,12 @@
 
 
 #ifndef lint
-static char *id_string="$Id: PSdev.c,v 1.5 1994/08/02 15:36:12 sander Exp $";
+static char *id_string="$Id: PSdev.c,v 1.8 1995/02/08 11:11:14 sander Exp $";
 #endif
 
 
 /*
- *   Copyright (C) 1993, 1994 by Georg Sander, Iris Lemke, and
+ *   Copyright (C) 1993--1995 by Georg Sander, Iris Lemke, and
  *                               the Compare Consortium 
  *
  *  This program and documentation is free software; you can redistribute 
@@ -48,6 +48,20 @@ static char *id_string="$Id: PSdev.c,v 1.5 1994/08/02 15:36:12 sander Exp $";
 
 /* 
  * $Log: PSdev.c,v $
+ * Revision 1.8  1995/02/08  11:11:14  sander
+ * Distribution version 1.3.
+ *
+ * Revision 1.7  1994/12/23  18:12:45  sander
+ * Manhatten layout added.
+ * Option interface cleared.
+ * infobox behaviour improved.
+ * First version of fisheye (carthesian).
+ * Options Noedge and nonode.
+ * Titles in the node title box are now sorted.
+ *
+ * Revision 1.6  1994/11/25  15:43:29  sander
+ * Printer interface added to allow to use VCG as a converter.
+ *
  * Revision 1.5  1994/08/02  15:36:12  sander
  * Bounding box for landscape format corrected.
  *
@@ -83,8 +97,8 @@ static char *id_string="$Id: PSdev.c,v 1.5 1994/08/02 15:36:12 sander Exp $";
  ************************************************************************/
 
 
-/* Mapping of global names of draw.c and drawlib.c
- * ===============================================
+/* Mapping of global names of draw.c, drawstr.c, drawlib.c
+ * =======================================================
  */
 
 #define INCLUDE_DRAW
@@ -107,7 +121,6 @@ static char *id_string="$Id: PSdev.c,v 1.5 1994/08/02 15:36:12 sander Exp $";
 #define gs_reverttextbox     ps_reverttextbox 
 #define gs_halfreverttextbox ps_halfreverttextbox  
 #define gs_stringbox         ps_stringbox         
-#define gs_infobox           ps_infobox       
 #define gs_calcrhombsize     ps_calcrhombsize 
 #define gs_rhomb             ps_rhomb        
 #define gs_revertrhomb       ps_revertrhomb 
@@ -127,11 +140,12 @@ static char *id_string="$Id: PSdev.c,v 1.5 1994/08/02 15:36:12 sander Exp $";
 #define gs_rectangle         ps_rectangle
 #define gs_actbackground     ps_actbackground
 
-#ifdef DRAWLIB_TOO_LARGE
-#define mysetto              ps_mysetto      
-#define mymoveto             ps_mymoveto       
-#define gs_draw_char         ps_draw_char      
-#endif
+#define mystretch            ps_mystretch
+#define myshrink             ps_myshrink
+#define myxpos               ps_myxpos
+#define myypos               ps_myypos
+#define gs_stlimit           ps_stlimit
+#define gs_shlimit           ps_shlimit
 
 
 /* Includes, Prototypes, Variables
@@ -159,9 +173,9 @@ float tm, int   form, int col, int numpag));
 
 static FILE *check_and_open_file  _PP((char *fname));
 
-static void ps_line            
+void ps_line            
 		_PP((int fx,int fy,int tx,int ty,int c));
-static void ps_thickline 
+void ps_thickline 
 		_PP((int fx,int fy,int tx,int ty,int t,int c));
 static void ps_dashedthickline 
 		_PP((int fx,int fy,int tx,int ty,int t,int c,int d));
@@ -177,6 +191,8 @@ static void ps_filltriangle
 		_PP((int x,int y,int w,int h,int c));
 static void ps_fillellipse 
 		_PP((int x,int y,int w,int h,int c));
+static void ps_freearrowpoint
+		_PP((int x1,int y1,int rx1,int ry1,int rx2,int ry2,int c));
 static void ps_w_arrowpoint
 		_PP((int x,int y,int size,int c));
 static void ps_s_arrowpoint
@@ -201,6 +217,10 @@ static void landscape_header
 
 static void common_header _PP((void));
 
+static void init_postscriptpage _PP((void));
+static void calc_paper_width	_PP((void));
+
+static void ps_setcolor _PP((int c));
 
 
 
@@ -224,6 +244,7 @@ extern int   act_output_type;
 extern int   act_color_type;
 extern int   act_ori_type;
 extern int   act_paper_num;
+extern int   act_paper_type;
 extern float act_width;
 extern float act_height;
 extern float act_left_margin;
@@ -233,11 +254,19 @@ extern float act_bottom_margin;
 extern int   act_xdpi;
 extern int   act_ydpi;
 extern float act_scaling;
+extern int   act_bbox;
 
 /* Counter for the wait message */
 
 static int wait_cntr;
 
+static int mreg_xmin;
+static int mreg_xmax;
+static int mreg_ymin;
+static int mreg_ymax;
+static int maxxdist;
+static int maxydist;
+static int maxover;
 
 
 /* Modules
@@ -258,9 +287,13 @@ static int wait_cntr;
  *  the measures, and output the graph. 
  */ 
 
+#ifdef ANSI_C
+void print_all_ps(char *fname, int a, int b, int c, int d)
+#else
 void print_all_ps(fname, a, b, c, d)
 char *fname;
 int a, b, c, d;
+#endif
 {
 	int form;
 	float scale, lm, bm, tm;	
@@ -271,6 +304,10 @@ int a, b, c, d;
 	wait_cntr  = 0;
 	if (a>c)  { form = a; a = c; c = form; }
 	if (b>d)  { form = b; b = d; d = form; }
+	mreg_xmin = a;
+	mreg_xmax = c;
+	mreg_ymin = b;
+	mreg_ymax = d;
 
 	switch (act_ori_type) {
 	case 1: form = 0; break;
@@ -282,11 +319,6 @@ int a, b, c, d;
 	bm = act_bottom_margin/2.54; 
 	tm = act_top_margin/2.54; 
 
-	if (act_paper_num > 1) {
-		c = (c-a)/act_paper_num + a;
-		d = (d-b)/act_paper_num + b;
-	}
-
 	switch (act_color_type) {
 	case 1: print_ps(fname, a, b, c, d,scale,lm,bm,tm,form,2,act_paper_num);
 		break;
@@ -339,6 +371,7 @@ int numpag;
 	int Vs_xmin, Vs_ymin, Vs_xmax, Vs_ymax;
 	int s_stretch, s_shrink, oldcol;
 	int i, j;
+	int oldx, oldy;
 
 	mfilename = fname;
 	bps_flag = col;
@@ -346,6 +379,14 @@ int numpag;
 	if (c<=a) return(0);
 	if (d<=b) return(0);
 
+	if (act_paper_num > 1) {
+		lm = 1.0/2.54; 
+		bm = 1.0/2.54; 
+		tm = 1.0/2.54; 
+		calc_paper_width();
+		act_bbox = 0;
+	}
+
 	fptr = check_and_open_file(fname);
 	if (fptr==NULL)  return(0);
 
@@ -372,12 +413,49 @@ int numpag;
 	if (form == 1) 	landscape_header(a, b, c, d, scale, lm, tm);
 	else		portrait_header( a, b, c, d, scale, lm, bm);
 
+	oldx = mreg_xmin;
 	for (i=1; i<=numpag; i++) {
+		if (numpag!=1) {
+			a = oldx;
+			if (a>=mreg_xmax) break;
+			c = a + maxxdist + maxover;
+			oldx = a + maxxdist;
+			if (c>=mreg_xmax) {
+				oldx = mreg_xmax;
+				c = mreg_xmax;
+			}
+			oldy = mreg_ymin;
+		}
 		for (j=1; j<=numpag; j++) {
 
 			gs_wait_message('y');
+
+			if (numpag==1) {
+				a = mreg_xmin;
+				c = mreg_xmax;
+				b = mreg_ymin;
+				d = mreg_ymax;
+			}
+			else {
+				b = oldy;
+				if (b>=mreg_ymax) break;
+
+				d = b + maxydist + maxover;
+				oldy = b + maxydist;
+				if (d>=mreg_ymax) {
+					oldy = mreg_ymax;
+					d = mreg_ymax;
+				}
+			}
+			V_xmin = a;
+			V_xmax = c;
+			V_ymin = b;
+			V_ymax = d;
+			dist = V_ymax-V_ymin;
+
 			FPRINTF(fptr,"%%%%Page: %d %d\n",
 				(i-1)*numpag+j, (i-1)*numpag+j);
+			/* if (numpag>1) init_postscriptpage(); */
 			FPRINTF(fptr,"72 %3.2f div dup scale\n",scale);
 			if (numpag == 1) {
 				if (form == 1) {	
@@ -408,36 +486,20 @@ int numpag;
 
 			/* draw to file */
 	
-			if (form==1) {
-				V_xmin = a+(i-1)*(c-a);     
-				if (j>1) V_ymin = b+(j-1)*(d-b)-scale;
-				else	 V_ymin = b+(j-1)*(d-b);
-				if (i<numpag) V_xmax = a+i*(c-a)+scale;     
-				else          V_xmax = a+i*(c-a);     
-				V_ymax = b+j*(d-b);
+			if (colored) {
+				ps_rectangle(V_xmin, V_ymin, V_xmax-V_xmin, V_ymax-V_ymin, G_color);
 			}
-			else {
-				V_xmin = a+(i-1)*(c-a);     
-				V_ymin = b+(j-1)*(d-b);
-				if (i<numpag) V_xmax = a+i*(c-a)+scale;     
-				else          V_xmax = a+i*(c-a);     
-				if (j<numpag) V_ymax = b+j*(d-b)+scale;
-				else          V_ymax = b+j*(d-b);
+			else {	
+				ps_rectangle(V_xmin, V_ymin, V_xmax-V_xmin, V_ymax-V_ymin, WHITE);
 			}
-			dist = V_ymax-V_ymin;
-
-			if (colored)
-				ps_rectangle(a, b, c-a+1, d-b+1, G_color);
-			else	
-				ps_rectangle(a, b, c-a+1, d-b+1, WHITE);
 			ps_draw_main();
 
 			FPRINTF(fptr,"showpage\n");
 			FPRINTF(fptr,"%%%%PageTrailer\n");
 		}
 	}
-	FPRINTF(fptr,"%%Trailer\n");
-	FPRINTF(fptr,"%%EOF\n");
+	FPRINTF(fptr,"%%%%Trailer\n");
+	FPRINTF(fptr,"%%%%EOF\n");
 
 	if (fptr) fclose(fptr);
 	fptr = NULL;
@@ -462,8 +524,12 @@ int numpag;
  * On error, we return NULL, otherwise the opend file.
  */
 
+#ifdef ANSI_C
+static FILE *check_and_open_file(char *fname)
+#else
 static FILE *check_and_open_file(fname)
 char *fname;
+#endif
 {
 	FILE *f;
         char *c;
@@ -471,7 +537,11 @@ char *fname;
         c = fname;
         while (*c) c++;
         if (c>fname) c--;
+#ifdef VMS
+        if (*c==']') {
+#else
         if (*c=='/') {
+#endif
                 return(NULL);
         }
 
@@ -513,25 +583,28 @@ float scale, lm, bm;
 	numyunits = (float)(d-b);
 
 	FPRINTF(fptr,"%%!PS-Adobe-2.0 EPSF-1.2\n");
-	FPRINTF(fptr,"%%%%Creator: %s\n", short_banner);
+	FPRINTF(fptr,"%%%%Creator: %s (P)\n", short_banner);
 	FPRINTF(fptr,"%%%%Title: %s\n", mfilename);
 	FPRINTF(fptr,"%%%%RCS-info: $I");
 	FPRINTF(fptr,"d$\n");
         FPRINTF(fptr,"%%%%SCCS-info: %%W");
         FPRINTF(fptr,"%% %%E");
         FPRINTF(fptr,"%%\n");
-	FPRINTF(fptr,"%%%%BoundingBox: %d %d %d %d\n",
+	if (act_bbox) FPRINTF(fptr,"%%%%BoundingBox: ");
+	else          FPRINTF(fptr,"%%%%ProposedBoundings: ");
+	FPRINTF(fptr," %d %d %d %d\n",
 		gstoint(lm*72),gstoint(bm*72),
 		gstoint(lm*72)+gstoint(72*numxunits/scale),
 		gstoint(bm*72)+gstoint(72*numyunits/scale));
+
 	FPRINTF(fptr,"%%%%EndComments\n");
 	
 	common_header();
 }
 
 
-/*  Portrait format
- *  ---------------
+/*  Landscape format
+ *  ----------------
  */
 
 #ifdef ANSI_C           
@@ -551,14 +624,16 @@ float scale, lm, tm;
 	numyunits = (float)(d-b);
 
 	FPRINTF(fptr,"%%!PS-Adobe-2.0 EPSF-1.2\n");
-	FPRINTF(fptr,"%%%%Creator: %s\n", short_banner);
+	FPRINTF(fptr,"%%%%Creator: %s (L)\n", short_banner);
 	FPRINTF(fptr,"%%%%Title: %s\n", mfilename);
 	FPRINTF(fptr,"%%%%RCS-info: $I");
 	FPRINTF(fptr,"d$\n");
         FPRINTF(fptr,"%%%%SCCS-info: %%W");
         FPRINTF(fptr,"%% %%E");
         FPRINTF(fptr,"%%\n");
-	FPRINTF(fptr,"%%%%BoundingBox: %d %d %d %d\n",
+	if (act_bbox) FPRINTF(fptr,"%%%%BoundingBox: ");
+	else          FPRINTF(fptr,"%%%%ProposedBoundings: ");
+	FPRINTF(fptr," %d %d %d %d\n",
 		gstoint(tm*72),gstoint(lm*72),
 		gstoint(tm*72)+gstoint(72*numyunits/scale),
 		gstoint(lm*72)+gstoint(72*numxunits/scale));
@@ -571,7 +646,11 @@ float scale, lm, tm;
 /*  Common Header for both formats
  */
 
+#ifdef ANSI_C
+static void common_header(void)
+#else
 static void common_header()
+#endif
 {
 	FPRINTF(fptr,"1 setlinecap\n");
 	FPRINTF(fptr,"1 setlinejoin\n");
@@ -622,14 +701,94 @@ static void common_header()
 }
 
 
+/*  Initialize a page, i.e. fill the whole page with the background color
+ *  ---------------------------------------------------------------------
+ */
+
+#ifdef ANSI_C
+static void init_postscriptpage(void)
+#else
+static void init_postscriptpage()
+#endif
+{
+	int a, b, h;
+
+	switch (act_paper_type) {
+	case 1: a = 595;  b = 850;  break;
+	case 2: a = 525;  b = 765;  break;
+	case 3: a = 425;  b = 595;  break;
+	case 4: a = 792;  b = 1224; break;
+	case 5: a = 612;  b = 792;  break;
+	case 6: a = 612;  b = 1008; break;
+	}
+
+	if (colored) ps_setcolor(G_color);
+	else ps_setcolor(WHITE);
+
+	FPRINTF(fptr,"NP %d %d M %d %d P %d %d P %d %d P CP fill\n",
+		0,   0,  0,  b,  a,  b,  a,  0);
+}
+
+
+
+/*  Calculate the maximal number of pixels on a page 
+ *  ------------------------------------------------
+ *  This is used for multipage output only.
+ */
+
+#ifdef ANSI_C
+static void calc_paper_width(void)
+#else
+static void calc_paper_width()
+#endif
+{
+	float mx, my;
+
+	assert((act_paper_num>1));
+        switch (act_paper_type) {
+        case 1:
+                if (act_ori_type==1)    { mx = 21.0; my = 30.0; }
+                else                    { mx = 30.0; my = 21.0; }
+                break;
+        case 2:
+                if (act_ori_type==1)    { mx = 18.5; my = 27.0; }
+                else                    { mx = 27.0; my = 18.5; }
+                break;
+        case 3:
+                if (act_ori_type==1)    { mx = 15.0; my = 21.0; }
+                else                    { mx = 21.0; my = 15.0; }
+                break;
+        case 4:
+                if (act_ori_type==1)    { mx = 27.94; my = 43.18; }
+                else                    { mx = 43.18; my = 27.94; }
+                break;
+        case 5:
+                if (act_ori_type==1)    { mx = 21.59; my = 27.94; }
+                else                    { mx = 27.94; my = 21.59; }
+                break;
+        case 6:
+                if (act_ori_type==1)    { mx = 21.59; my = 35.56; }
+                else                    { mx = 35.56; my = 21.59; }
+                break;
+        }
+        mx = (mx-3.0);
+        my = (my-3.0);
+	maxxdist = (int)(mx * act_xdpi / act_scaling / 2.54);
+	maxydist = (int)(my * act_ydpi / act_scaling / 2.54);
+	maxover =  (int)(1.0 * act_ydpi / act_scaling / 2.54);
+}
 
 
 /*--------------------------------------------------------------------*/
 /*  Device driver for pbm and ppm Files                               */
 /*--------------------------------------------------------------------*/
 
+#ifdef ANSI_C
+static void 	ps_setcolor(int c)
+#else
 static void 	ps_setcolor(c)
 int c; 
+#endif
 {
 	float col;
 
@@ -658,8 +817,12 @@ int c;
  * at (x,y) with width w, height h and color c
  */
 
+#ifdef ANSI_C
+static void    ps_rectangle(int x,int y,int w,int h,int c)
+#else
 static void    ps_rectangle(x, y, w, h, c)
 int     x, y, w, h, c;
+#endif
 {
 	ps_setcolor(c);
 
@@ -676,8 +839,12 @@ int     x, y, w, h, c;
  * at (x,y) with width w, height h and color c
  */
 
+#ifdef ANSI_C
+static void    ps_fillrhomb(int x,int y,int w,int h,int c)
+#else
 static void    ps_fillrhomb(x, y, w, h, c)
 int     x, y, w, h, c;
+#endif
 {
 	ps_setcolor(c);
 
@@ -694,8 +861,12 @@ int     x, y, w, h, c;
  * at (x,y) with width w, height h and color c
  */
 
+#ifdef ANSI_C
+static void    ps_filltriangle(int x,int y,int w,int h,int c)
+#else
 static void    ps_filltriangle(x, y, w, h, c)
 int     x, y, w, h, c;
+#endif
 {
 	ps_setcolor(c);
 
@@ -733,8 +904,12 @@ int     x, y, w, h, c;
  * at (x,y) with width w, height h and color c
  */
 
+#ifdef ANSI_C
+static void    ps_fillellipse(int x,int y,int w,int h,int c)
+#else
 static void    ps_fillellipse(x, y, w, h, c)
 int     x, y, w, h, c;
+#endif
 {
         int i,j;
 
@@ -779,8 +954,12 @@ int     x, y, w, h, c;
  * from fx,fy to tx,ty with color c.
  */
 
-static void    ps_line(fx, fy, tx, ty, c)
+#ifdef ANSI_C
+void    ps_line(int fx,int fy,int tx,int ty,int c)
+#else
+void    ps_line(fx, fy, tx, ty, c)
 int     fx, fy, tx, ty, c;
+#endif
 {
 	ps_setcolor(c);
 
@@ -796,8 +975,12 @@ int     fx, fy, tx, ty, c;
  * from fx,fy to tx,ty with color c.
  */
 
-static void    ps_thickline(fx, fy, tx, ty, t, c)
+#ifdef ANSI_C
+void    ps_thickline(int fx,int fy,int tx,int ty,int t,int c)
+#else
+void    ps_thickline(fx, fy, tx, ty, t, c)
 int     fx, fy, tx, ty, t, c;
+#endif
 {
         ps_setcolor(c);
 
@@ -813,8 +996,12 @@ int     fx, fy, tx, ty, t, c;
  * from fx,fy to tx,ty with color c.
  */
 
+#ifdef ANSI_C
+static void    ps_dashedthickline(int fx,int fy,int tx,int ty,int t,int c,int d)
+#else
 static void    ps_dashedthickline(fx, fy, tx, ty, t, c, d)
 int     fx, fy, tx, ty, t, c, d;
+#endif
 {
         ps_setcolor(c);
 
@@ -830,8 +1017,12 @@ int     fx, fy, tx, ty, t, c, d;
  * from fx,fy to tx,ty with color c.
  */
 
+#ifdef ANSI_C
+static void    ps_dottedthickline(int fx,int fy,int tx,int ty,int t,int c,int d)
+#else
 static void    ps_dottedthickline(fx, fy, tx, ty, t, c, d)
 int     fx, fy, tx, ty, t, c, d;
+#endif
 {
         ps_setcolor(c);
 
@@ -847,8 +1038,14 @@ int     fx, fy, tx, ty, t, c, d;
  * with color c and mode m.
  */
 
+#ifdef ANSI_C
+static void ps_bezierspline(
+	int x0,int y0,int x1,int y1,int x2,int y2,int x3,int y3,
+	int t,int c,int m)
+#else
 static void ps_bezierspline(x0,y0,x1,y1,x2,y2,x3,y3,t,c,m)
 int x0,y0,x1,y1,x2,y2,x3,y3,t,c,m;
+#endif
 {
 	int d;
 
@@ -879,13 +1076,40 @@ int x0,y0,x1,y1,x2,y2,x3,y3,t,c,m;
 }
 
 
+/* Draw a free arrowhead
+ * ---------------------
+ * i.e. draw a triangle between (x1, y1), (rx1, ry1), (rx2, ry2)
+ */
+
+
+#ifdef ANSI_C
+static void ps_freearrowpoint(int x1,int y1,int rx1,int ry1,int rx2,int ry2,
+		int c)
+#else
+static void ps_freearrowpoint(x1, y1, rx1, ry1, rx2, ry2, c)
+int x1, y1, rx1, ry1, rx2, ry2, c;
+#endif
+{
+	ps_setcolor(c);
+	FPRINTF(fptr,"NP %d %d M %d %d P %d %d P CP fill\n",
+		x1-V_xmin, dist - (y1-V_ymin),
+		rx1-V_xmin,dist - (ry1-V_ymin),
+		rx2-V_xmin,dist - (ry2-V_ymin));
+}
+
+
+
 /* Draw an arrowhead to the north
  * ------------------------------
  * with `size' of katheses and color
  */
 
+#ifdef ANSI_C
+static void ps_n_arrowpoint(int x,int y,int size,int c)
+#else
 static void ps_n_arrowpoint(x, y, size, c)
 int     x, y, size, c;
+#endif
 {
 	int k;
 
@@ -905,8 +1129,12 @@ int     x, y, size, c;
  * with `size' of katheses and color
  */
 
+#ifdef ANSI_C
+static void ps_s_arrowpoint(int x,int y,int size,int c)
+#else
 static void ps_s_arrowpoint(x, y, size, c)
 int     x, y, size, c;
+#endif
 {
 	int k;
 
@@ -927,8 +1155,12 @@ int     x, y, size, c;
  * with `size' of katheses and color
  */
 
+#ifdef ANSI_C
+static void ps_e_arrowpoint(int x,int y,int size,int c)
+#else
 static void ps_e_arrowpoint(x, y, size, c)
 int     x, y, size, c;
+#endif
 {
 	int k;
 
@@ -949,8 +1181,12 @@ int     x, y, size, c;
  * with `size' of katheses and color
  */
 
+#ifdef ANSI_C
+static void ps_w_arrowpoint(int x,int y,int size,int c)
+#else
 static void ps_w_arrowpoint(x, y, size, c)
 int     x, y, size, c;
+#endif
 {
 	int k;
 
@@ -970,8 +1206,12 @@ int     x, y, size, c;
  * with `size' of katheses and color
  */
 
+#ifdef ANSI_C
+static void ps_ne_arrowpoint(int x,int y,int size,int c)
+#else
 static void ps_ne_arrowpoint(x, y, size, c)
 int     x, y, size, c;
+#endif
 {
 	ps_setcolor(c);
 
@@ -987,8 +1227,12 @@ int     x, y, size, c;
  * with `size' of katheses and color
  */
 
+#ifdef ANSI_C
+static void ps_nw_arrowpoint(int x,int y,int size,int c)
+#else
 static void ps_nw_arrowpoint(x, y, size, c)
 int     x, y, size, c;
+#endif
 {
 	ps_setcolor(c);
 
@@ -1004,8 +1248,12 @@ int     x, y, size, c;
  * with `size' of katheses and color
  */
 
+#ifdef ANSI_C
+static void ps_se_arrowpoint(int x,int y,int size,int c)
+#else
 static void ps_se_arrowpoint(x, y, size, c)
 int     x, y, size, c;
+#endif
 {
 	ps_setcolor(c);
 
@@ -1021,8 +1269,12 @@ int     x, y, size, c;
  * with `size' of katheses and color
  */
 
+#ifdef ANSI_C
+static void ps_sw_arrowpoint(int x,int y,int size,int c)
+#else
 static void ps_sw_arrowpoint(x, y, size, c)
 int     x, y, size, c;
+#endif
 {
 	ps_setcolor(c);
 
