@@ -18,11 +18,11 @@
 /*--------------------------------------------------------------------*/
 
 #ifndef lint
-static char *id_string="$Id: X11devfs.c,v 1.3 1994/08/03 13:58:44 sander Exp $";
+static char *id_string="$Id: X11devfs.c,v 1.7 1995/02/09 20:44:22 sander Exp $";
 #endif
 
 /*
- *   Copyright (C) 1993, 1994 by Georg Sander, Iris Lemke, and
+ *   Copyright (C) 1993--1995 by Georg Sander, Iris Lemke, and
  *                               the Compare Consortium 
  *
  *  This program and documentation is free software; you can redistribute 
@@ -46,6 +46,23 @@ static char *id_string="$Id: X11devfs.c,v 1.3 1994/08/03 13:58:44 sander Exp $";
 
 /* 
  * $Log: X11devfs.c,v $
+ * Revision 1.7  1995/02/09  20:44:22  sander
+ * Call of getwd removed.
+ *
+ * Revision 1.6  1995/02/08  11:11:14  sander
+ * Distribution version 1.3.
+ *
+ * Revision 1.5  1994/12/23  18:12:45  sander
+ * Manhatten layout added.
+ * Option interface cleared.
+ * infobox behaviour improved.
+ * First version of fisheye (carthesian).
+ * Options Noedge and nonode.
+ * Titles in the node title box are now sorted.
+ *
+ * Revision 1.4  1994/11/23  14:50:47  sander
+ * Drawing speed improved by removing some superfluous XSyncs.
+ *
  * Revision 1.3  1994/08/03  13:58:44  sander
  * Horizontal order mechanism changed.
  * Attribute horizontal_order for edges added.
@@ -93,15 +110,30 @@ static char *id_string="$Id: X11devfs.c,v 1.3 1994/08/03 13:58:44 sander Exp $";
 #include <X11/cursorfont.h>
 #include <X11/Xproto.h>
 #include <ctype.h>
+#ifdef VMS
+#include <unixio.h>
+#include <unixlib.h>
+#include <processes.h>
+#include <file.h>
+#include <descrip.h>
+#include <rmsdef.h>
+#include <rms.h>
+#include <types.h>
+#include <ssdef.h>
+#include <stat.h>
+#define STAT stat
+#else
 #include <sys/types.h> 
 #include <sys/stat.h> 
 #include <sys/param.h> 
 #include <dirent.h> 
-#include <time.h> 
 #include <grp.h> 
 #include <pwd.h> 
+#endif
+#include <time.h> 
 #include <string.h> 
 #include <math.h>
+#include "options.h"
 #include "main.h"
 #include "grammar.h"
 #include "X11devfs.h"
@@ -165,8 +197,11 @@ typedef struct gs_scroll_bar {
  * ==========
  */
 
-void gs_exit	_PP((int x));
+void    gs_exit         _PP((int x));
+void    do_gblExpose    _PP((XEvent *eventp));
+void    do_gblConfigureNotify  _PP((XEvent *eventp));
 
+ 
 static void   allocateDialogBox         _PP((void));
 static Window allocate_selection_point  _PP((int x,int y));
 static Window allocate_button		_PP((int x,int y));
@@ -389,9 +424,13 @@ static int dialog_box_there = 0;
  *  This is done once at the beginning.
  */
 
+#ifdef ANSI_C
+void 	x11_init_fisel_dialog(Display *rd, int rs)
+#else
 void 	x11_init_fisel_dialog(rd, rs)
 Display *rd;
 int rs;
+#endif
 {
 	if (dialog_box_there) return;
 	root_display = rd;
@@ -413,9 +452,13 @@ int rs;
  *   This is done once on ExitDevice.
  */
 
+#ifdef ANSI_C
+void	x11_destroy_fisel_dialog(Display *rd, int rs)
+#else
 void	x11_destroy_fisel_dialog(rd, rs)
 Display *rd;
 int rs;
+#endif
 {
 	if (!dialog_box_there) return;
 	XDestroyWindow(root_display,dialog_window);
@@ -430,8 +473,13 @@ int rs;
  */
 
 static Window last_tmb;
+static char x11_win_label[1024];
 
+#ifdef ANSI_C
+static void allocateDialogBox(void)
+#else
 static void allocateDialogBox()
+#endif
 {
 	Window rret1, rret2;
 	XSetWindowAttributes attr;
@@ -470,7 +518,11 @@ static void allocateDialogBox()
 	hints.x = xpos;
 	hints.y = ypos;
 
-	XStoreName(root_display, dialog_window, "VCG File");
+        strcpy(x11_win_label, "VCG ");
+        strcat(x11_win_label, Dataname);
+        strcat(x11_win_label, " File");
+
+	XStoreName(root_display, dialog_window, x11_win_label);
 	XSetNormalHints(root_display, dialog_window, &hints);
 
         path_window = XCreateWindow(root_display, dialog_window,
@@ -533,8 +585,12 @@ static void allocateDialogBox()
  *  ---------------------------------------------------
  */
 
+#ifdef ANSI_C
+static Window allocate_selection_point(int x,int y)
+#else
 static Window allocate_selection_point(x,y)
 int x, y;
+#endif
 {
 	return(allocate_general_button(x,y,POINT_W,POINT_W));
 }
@@ -545,8 +601,12 @@ int x, y;
  *  -------------------------
  */
 
+#ifdef ANSI_C
+static Window allocate_button(int x,int y)
+#else
 static Window allocate_button(x,y)
 int x, y;
+#endif
 {
 	return(allocate_general_button(x,y,BUTTON_W,25));
 }
@@ -557,8 +617,12 @@ int x, y;
  *  ----------------------------------
  */
 
+#ifdef ANSI_C
+static Window allocate_general_button(int x,int y,int w,int h)
+#else
 static Window allocate_general_button(x,y,w,h)
 int x, y, w, h;
+#endif
 {
 	Window ww;
 	XSetWindowAttributes attr;
@@ -594,8 +658,12 @@ int x, y, w, h;
  *  ---------------------
  */
 
+#ifdef ANSI_C
+static Window allocate_scroll_bar(int x,int y,int len)
+#else
 static Window allocate_scroll_bar(x,y,len)
 int x, y, len;
+#endif
 {
 	Window w, wt;
 	XSetWindowAttributes attr;
@@ -638,11 +706,25 @@ int x, y, len;
 /*--------------------------------------------------------------------*/
 
 static int box_exposed;
+static int first_exposed = 1;
 
+#ifdef ANSI_C
+char *x11_print_fisel_dialog(void)
+#else
 char *x11_print_fisel_dialog()
+#endif
 {
 	XEvent event;
-	int i;
+        int i, x, y;
+        Window rret1, rret2;
+        unsigned int j;
+
+        if (first_exposed) {
+                first_exposed = 0;
+                (void)XQueryPointer(root_display,RootWindow(root_display, root_screen),
+                        &rret1, &rret2,&x, &y, &i, &i, &j);
+                XMoveWindow(root_display,dialog_window, x, y);
+        }
 
 	old_selection = -1;
 	act_fisel[0]  = 0;
@@ -652,7 +734,7 @@ char *x11_print_fisel_dialog()
 	sel_mode = presel_mode = 0;
 	for (i=0; i<MAX_RADIO_POINTS; i++)
 		radio_point[i].is = *(radio_point[i].glob);
-	if (!getwd(act_path)) act_path[0]=0;
+        if (!getcwd(act_path,MAXPATHLEN-1)) act_path[0]=0;
 	SPRINTF(act_fpath,"Enter path: %s_",act_path);
 	vscrollbar[0].is = 0;
 	rescan_fnames();
@@ -715,7 +797,11 @@ char *x11_print_fisel_dialog()
 	if (act_fpath[i-1]=='_') i--;
 	act_fpath[i]=0;
 
+#ifdef VMS
+	SPRINTF(act_path,"%s%s",&(act_fpath[12]), &(act_fisel[16]) );
+#else
 	SPRINTF(act_path,"%s/%s",&(act_fpath[12]), &(act_fisel[16]) );
+#endif
 	if (dialog_success)  return(act_path); 
 	else		     return(NULL);
 }
@@ -730,7 +816,11 @@ char *x11_print_fisel_dialog()
  *  ---------------
  */
 
+#ifdef ANSI_C
+static void draw_dialog_box(void)
+#else
 static void draw_dialog_box()
+#endif
 {
 	int i;
 
@@ -817,7 +907,11 @@ static void draw_dialog_box()
 
 
 
+#ifdef ANSI_C
+static void draw_sizes(void)
+#else
 static void draw_sizes()
+#endif
 {
 	int i, k, yp;
 	char *c1,*c2,*c3,isdir;
@@ -880,13 +974,21 @@ static void draw_sizes()
 					c1 = act_path;
 					while (*c1++);
 					while (c1>act_path) {
+#ifdef VMS
+						if (*c1==']') break;
+#else
 						if (*c1=='/') break;
+#endif
 						c1--;
 					}
 					*c1 = 0;
 				}
 				else {
+#ifdef VMS
+					strcat(act_path,"]");
+#else
 					strcat(act_path,"/");
+#endif
 					strcat(act_path,c3);
 				}
 				SPRINTF(act_fpath,"Enter path: %s_",act_path);
@@ -937,12 +1039,14 @@ static void draw_sizes()
 		yp += 18;
 	}
 	draw_textwin();
-	
-	XSync(root_display, 0);
 }
 
 
+#ifdef ANSI_C
+static void draw_textwin(void)
+#else
 static void draw_textwin()
+#endif
 {
 	int i;
 
@@ -993,9 +1097,13 @@ static void draw_textwin()
  *  ------------------------------------------------
  */
 
+#ifdef ANSI_C
+static void draw_text(int x,int y,char *m)
+#else
 static void draw_text(x,y, m)
 int x, y;
 char *m;
+#endif
 {
 	XSetForeground(root_display,
 		XDefaultGC(root_display, root_screen),
@@ -1011,9 +1119,13 @@ char *m;
  *  -----------------------------------------------------------
  */
 
+#ifdef ANSI_C
+static void draw_underl_text(int x,int y,char *m)
+#else
 static void draw_underl_text(x,y, m)
 int x, y;
 char *m;
+#endif
 {
 	int len;
 
@@ -1035,9 +1147,13 @@ char *m;
  *  ------------------------------
  */
 
+#ifdef ANSI_C
+static void draw_dialog_point(Window w, int color)
+#else
 static void draw_dialog_point(w, color)
 Window w;
 int color;
+#endif
 {
 	if (color==0) {
 		XSetForeground(root_display,
@@ -1053,7 +1169,7 @@ int color;
 			XDefaultGC(root_display, root_screen),
 			1,1,
 			POINT_W-2,POINT_W-2);
-	XSync(root_display, 0);
+	/* XSync(root_display, 0); */
         XFlush(root_display);
 }
 
@@ -1062,8 +1178,12 @@ int color;
  * ---------------------------
  */
 
+#ifdef ANSI_C
+static void normalize_point(Window w)
+#else
 static void normalize_point(w)
 Window w;
+#endif
 {
         XWindowAttributes retWinAttr;
 
@@ -1088,7 +1208,11 @@ Window w;
  * =============================
  */
 
+#ifdef ANSI_C
+static void dialog_main_loop(void)
+#else
 static void dialog_main_loop()
+#endif
 {
 	XEvent event;
 
@@ -1170,9 +1294,13 @@ static void dialog_main_loop()
  * ---------------------------------------
  */
 
+#ifdef ANSI_C
+static void prologue (XEvent *eventp, char *event_name)
+#else
 static void prologue (eventp, event_name)
 XEvent *eventp;
 char *event_name;
+#endif
 {
 	XAnyEvent *e = (XAnyEvent *) eventp;
 
@@ -1188,8 +1316,12 @@ char *event_name;
  * ---------------
  */
 
+#ifdef ANSI_C
+static void do_KeyPress (XEvent *eventp)
+#else
 static void do_KeyPress (eventp)
 XEvent *eventp;
+#endif
 {
     	XKeyEvent *e = (XKeyEvent *) eventp;
     	KeySym ks;
@@ -1382,8 +1514,12 @@ XEvent *eventp;
 
 static Window act_scroll_window = 0;
  
+#ifdef ANSI_C
+static void do_ButtonPress (XEvent *eventp)
+#else
 static void do_ButtonPress (eventp)
 XEvent *eventp;
+#endif
 {
 	XButtonEvent *e = (XButtonEvent *) eventp;
 	Window w;
@@ -1466,8 +1602,12 @@ XEvent *eventp;
  * --------------
  */
 
+#ifdef ANSI_C
+static void do_ButtonRelease (XEvent *eventp)
+#else
 static void do_ButtonRelease (eventp)
 XEvent *eventp;
+#endif
 {
 	XButtonEvent *e = (XButtonEvent *) eventp;
 	int i;
@@ -1495,8 +1635,12 @@ XEvent *eventp;
  * ------------------------------------
  */
 
+#ifdef ANSI_C
+static void do_EnterNotify (XEvent *eventp)
+#else
 static void do_EnterNotify (eventp)
 XEvent *eventp;
+#endif
 {
     	XCrossingEvent *e = (XCrossingEvent *) eventp;
 	Window w;
@@ -1531,8 +1675,12 @@ XEvent *eventp;
  * The menupoint is not anymore selected.
  */ 
 
+#ifdef ANSI_C
+static void do_LeaveNotify (XEvent *eventp)
+#else
 static void do_LeaveNotify (eventp)
 XEvent *eventp;
+#endif
 {
     	XCrossingEvent *e = (XCrossingEvent *) eventp;
 	Window w;
@@ -1555,8 +1703,12 @@ XEvent *eventp;
  * ---------------
  */
 
+#ifdef ANSI_C
+static void do_Expose (XEvent *eventp)
+#else
 static void do_Expose (eventp)
 XEvent *eventp;
+#endif
 {
 	XExposeEvent *e = (XExposeEvent *) eventp;
 
@@ -1568,6 +1720,7 @@ XEvent *eventp;
 	}
 	else box_exposed = 0;
 	old_selection = -1;
+	do_gblExpose(eventp);
 }
 
 
@@ -1575,15 +1728,21 @@ XEvent *eventp;
  * -----------------------
  */
 
+#ifdef ANSI_C
+static void do_ConfigureNotify (XEvent *eventp)
+#else
 static void do_ConfigureNotify (eventp)
 XEvent *eventp;
+#endif
 {
 	XConfigureEvent *e = (XConfigureEvent *) eventp;
 
 	if (e->window==dialog_window) {
 		XResizeWindow(root_display,dialog_window,DIAL_W,DIAL_H);
-		/* if (!box_exposed) draw_dialog_box(); */
+		draw_dialog_box(); 
 	}
+        else do_gblConfigureNotify(eventp);
+
 	old_selection = -1;
 }
 
@@ -1592,8 +1751,12 @@ XEvent *eventp;
  * --------------------------
  */
 
+#ifdef ANSI_C
+static void do_MotionNotify (XEvent *eventp)
+#else
 static void do_MotionNotify (eventp)
 XEvent *eventp;
+#endif
 {
 	int i;
 
@@ -1615,8 +1778,12 @@ XEvent *eventp;
 /*--------------------------------------------------------------------*/
 
 
+#ifdef ANSI_C
+static void read_fisel_scrollbar(int val)
+#else
 static void read_fisel_scrollbar(val)
 int val;
+#endif
 {
 	if (val<0) val = 0;
 	if (val+SCROLLBAR_LEN >= vscrollbar[0].len) 
@@ -1626,7 +1793,11 @@ int val;
 }
 
 
+#ifdef ANSI_C
+static void set_fisel_scrollbar(void)
+#else
 static void set_fisel_scrollbar()
+#endif
 {
 	int val;
 
@@ -1647,19 +1818,50 @@ static void set_fisel_scrollbar()
  * ----------------------------------------------
  */
 
+#ifdef ANSI_C
+static int calc_dir_entries(void)
+#else
 static int calc_dir_entries()
+#endif
 {
+#ifdef VMS
+static 	char dirp[MAXPATHLEN];
+static 	char specif[255];
+	long context=0;
+	$DESCRIPTOR(lib_spec,specif);
+	$DESCRIPTOR(lib_result,dirp);
+#else
 	DIR *dirp;
 	struct dirent *dp;
+#endif
 	int result;
 	char *c;
-	struct stat   file_stat;
+	struct stat file_stat;
 
 	result = 0;
+#ifdef VMS
+	if (act_path[0]) SPRINTF(specif,"%s*.*;*",act_path);
+	else SPRINTF(specif,"[]*.*;*");
+#else
 	if (act_path[0]) dirp = opendir(act_path);
 	else dirp = opendir(".");
 	if (!dirp) return(0);
+#endif
 
+#ifdef VMS
+	while (lib$find_file(&lib_spec,&lib_result,&context) != RMS$_NMF) {
+		if (!stat(dirp,&file_stat)) {
+			if( ((file_stat.st_mode)&S_IFMT) == S_IFDIR ) {
+				result++;
+				continue;
+			}
+		}
+		c = dirp;
+		while (*c) c++;
+		while ((c>(dirp)) && (*c!=';')) c--;
+		*c='\0';
+		while ((c>(dirp)) && (*c!='.')) c--;
+#else
 	while ((dp = readdir(dirp))!=NULL) {
 		if (dp->d_name[0]!='.') {
 			SPRINTF(act_fpath,"%s/%s",act_path,dp->d_name);
@@ -1672,6 +1874,7 @@ static int calc_dir_entries()
 			c = dp->d_name;
 			while (*c) c++;
 			while ((c>(dp->d_name)) && (*c!='.')) c--;
+#endif
 			switch (presel_mode) {
 			case 0: result++; 
 				break;
@@ -1688,9 +1891,15 @@ static int calc_dir_entries()
 			        if (strcmp(c,".PPM")==0) result++;
 				break;
 			}
+#ifndef VMS
 		}
+#endif
 	}
+#ifdef VMS
+	lib$find_file_end(&context);
+#else
 	(void)closedir(dirp);
+#endif
 	return(result);
 }
 
@@ -1699,7 +1908,11 @@ static int calc_dir_entries()
  * --------------------------------------------
  */
 
+#ifdef ANSI_C
+static void alloc_fnames(void)
+#else
 static void alloc_fnames()
+#endif
 {
 	int newsize,i;
 
@@ -1713,8 +1926,8 @@ static void alloc_fnames()
 		if (act_fsort ) free(act_fsort );
 		if (act_dirtag) free(act_dirtag);
 		act_fbuf_size = newsize;
-		act_fnames = malloc(FNAMESIZE*newsize);
-		act_infos  = malloc(INFOSIZE *newsize);
+		act_fnames = (char *)malloc(FNAMESIZE*newsize);
+		act_infos  = (char *)malloc(INFOSIZE *newsize);
 		act_fsort  = (int *)malloc(sizeof(int)*newsize);
 		act_dirtag = (char*)malloc(sizeof(char)*newsize);
 		if (!act_fnames) Fatal_error("memory exhausted","");
@@ -1736,15 +1949,28 @@ static void alloc_fnames()
  * --------------------------------------------
  */
 
+#ifdef ANSI_C
+static void rescan_fnames(void)
+#else
 static void rescan_fnames()
+#endif
 {
+#ifdef VMS
+static 	char dirp[MAXPATHLEN];
+static 	char specif[255];
+	long context = 0;
+	$DESCRIPTOR(lib_spec,specif);
+	$DESCRIPTOR(lib_result,dirp);
+	char d_name[MAXPATHLEN];
+#else
 	DIR *dirp;
 	struct dirent *dp;
+#endif
 	int i,j,month,is_okay;
 	struct stat   file_stat;
-	struct group  *gid_stat;
-	struct passwd *uid_stat;
-	char *c;
+	struct group  *gid_stat = 0;
+	struct passwd *uid_stat = 0;
+	char *c = 0;
 
 	XDefineCursor(root_display,dialog_window,busyCursor); 
 	XSync(root_display, 0);
@@ -1753,7 +1979,9 @@ static void rescan_fnames()
 	i = 0;
 	while ((act_fpath[i]) && (i<MAXPATHLEN-3)) i++;
 	if (act_fpath[i-1]=='_') i--;
+#ifndef VMS
 	if (act_fpath[i-1]=='/') i--;
+#endif
 	act_fpath[i]=0;
 	SPRINTF(act_path,"%s",&(act_fpath[12]));
 
@@ -1765,14 +1993,31 @@ static void rescan_fnames()
 	/* Allocate memory to contain the directory */
 	alloc_fnames();
 
+#ifdef VMS
+	if (act_path[0]) SPRINTF(specif,"%s*.*;*",act_path);
+	else SPRINTF(specif,"[]*.*;*");
+#else
 	if (act_path[0]) dirp = opendir(act_path);
 	else dirp = opendir(".");
 	if (!dirp) { 
 		XDefineCursor(root_display,dialog_window,pointCursor);
 		return; 
 	}
+#endif
 
 	i = 0;
+#ifdef VMS
+	while (lib$find_file(&lib_spec,&lib_result,&context) != RMS$_NMF) {
+		if (!stat(dirp,&file_stat)) {
+			is_okay = 0;
+			if( ((file_stat.st_mode)&S_IFMT)==S_IFDIR ) is_okay = 1;
+			c = dirp;
+			while ( (*c) && (*c!=']') ) c++;
+			c++;
+			strcpy(d_name,c);
+			while (*c) c++;
+			while ((c>(dirp)) && (*c!='.')) c--;
+#else
 	while ((dp = readdir(dirp))!=NULL) {
 		if (dp->d_name[0]=='.') continue;
 		SPRINTF(act_fpath,"%s/%s",act_path,dp->d_name);
@@ -1782,27 +2027,39 @@ static void rescan_fnames()
 			c = dp->d_name;
 			while (*c) c++;
 			while ((c>(dp->d_name)) && (*c!='.')) c--;
+#endif
 			switch (presel_mode) {
 			case 0: is_okay = 1; break;
-			case 1: if (strcmp(c,".vcg")==0) is_okay = 1; 
-			        if (strcmp(c,".VCG")==0) is_okay = 1;
+			case 1: if (strncmp(c,".vcg",4)==0) is_okay = 1; 
+			        if (strncmp(c,".VCG",4)==0) is_okay = 1;
 				break;
-			case 2: if (strcmp(c,".ps")==0)  is_okay = 1;
-			        if (strcmp(c,".PS")==0)  is_okay = 1;
+			case 2: if (strncmp(c,".ps",4)==0)  is_okay = 1;
+			        if (strncmp(c,".PS",4)==0)  is_okay = 1;
 				break;
-			case 3: if (strcmp(c,".pbm")==0) is_okay = 1;
-			        if (strcmp(c,".PBM")==0) is_okay = 1;
+			case 3: if (strncmp(c,".pbm",4)==0) is_okay = 1;
+			        if (strncmp(c,".PBM",4)==0) is_okay = 1;
 				break;
-			case 4: if (strcmp(c,".ppm")==0) is_okay = 1;
-			        if (strcmp(c,".PPM")==0) is_okay = 1;
+			case 4: if (strncmp(c,".ppm",4)==0) is_okay = 1;
+			        if (strncmp(c,".PPM",4)==0) is_okay = 1;
 				break;
 			}
 			if (!is_okay) continue;
 			for (j=0; j<FNAMESIZE; j++) {
+#ifdef VMS
+				act_fnames[i*FNAMESIZE+j] = d_name[j];
+				if (d_name[j]==0) break;
+#else
 				act_fnames[i*FNAMESIZE+j] = dp->d_name[j];
 				if (dp->d_name[j]==0) break;
+#endif
 			}
+
+#ifdef VMS
+			if (((file_stat.st_mode)&S_IFMT)==S_IFDIR) 
+				act_dirtag[i] = 1;
+#else
 			if (S_ISDIR(file_stat.st_mode)) act_dirtag[i] = 1;
+#endif
 			switch (sel_mode) {
 			case 0: /* size  */
 				SPRINTF(&(act_infos[i*INFOSIZE]),"%12d",file_stat.st_size);
@@ -1810,6 +2067,21 @@ static void rescan_fnames()
 
 				break;
 			case 1: /* mode  */
+#ifdef VMS
+				if( ((file_stat.st_mode)&S_IFMT)==S_IFDIR ) 
+				     act_infos[i* INFOSIZE]='d';
+				else act_infos[i* INFOSIZE]='-';
+				if ((file_stat.st_mode)&S_IREAD) 
+				     act_infos[i* INFOSIZE+1]='r';
+				else act_infos[i* INFOSIZE+1]='-';
+				if ((file_stat.st_mode)&S_IWRITE) 
+			  	     act_infos[i* INFOSIZE+2]='w';
+				else act_infos[i* INFOSIZE+2]='-';
+				if ((file_stat.st_mode)&S_IEXEC) 
+				     act_infos[i* INFOSIZE+3]='e';
+				else act_infos[i* INFOSIZE+3]='-';
+				act_infos[i* INFOSIZE+4]=0;
+#else
 				if (S_ISDIR(file_stat.st_mode)) act_infos[i* INFOSIZE]='d';
 				else act_infos[i* INFOSIZE]='-';
 				if ((file_stat.st_mode)&S_IRUSR) act_infos[i* INFOSIZE+1]='r';
@@ -1831,10 +2103,15 @@ static void rescan_fnames()
 				if ((file_stat.st_mode)&S_IXOTH) act_infos[i* INFOSIZE+9]='x';
 				else act_infos[i* INFOSIZE+9]='-';
 				act_infos[i* INFOSIZE+10] = 0;
+#endif
 				break;
 			case 2: /* date  */
 				SPRINTF(act_fpath,"%s",
+#ifdef VMS
+					ctime(&(file_stat.st_ctime)));
+#else
 					asctime(gmtime(&(file_stat.st_ctime))));
+#endif
 				act_fpath[7]=0;
 				act_fpath[10]=0;
 				act_fpath[16]=0;
@@ -1861,20 +2138,26 @@ static void rescan_fnames()
 					&(act_fpath[11]));
 				break;
 			case 3: /* owner */
+#ifndef VMS
 				uid_stat = getpwuid(file_stat.st_uid);
 				if (uid_stat) 
 					SPRINTF(&(act_infos[i*INFOSIZE]),
 						"%s",uid_stat->pw_name);
-				else 	SPRINTF(&(act_infos[i*INFOSIZE]),
+				else 	
+#endif
+					SPRINTF(&(act_infos[i*INFOSIZE]),
 						"%12d",file_stat.st_uid);
 				break;
 			case 4: /* group */
+#ifndef VMS
 				gid_stat = getgrgid(file_stat.st_gid);
 
 				if (gid_stat)
 					SPRINTF(&(act_infos[i*INFOSIZE]),
 						"%s",gid_stat->gr_name);
-				else    SPRINTF(&(act_infos[i*INFOSIZE]),
+				else    
+#endif
+					SPRINTF(&(act_infos[i*INFOSIZE]),
 						"%12d",file_stat.st_gid);
 				break;
 			}
@@ -1884,7 +2167,11 @@ static void rescan_fnames()
 			if (i>max_fname) break;
 		}
 	}
+#ifdef VMS
+	lib$find_file_end(&context);
+#else
 	(void)closedir(dirp);
+#endif
 
 	/* Sort the entries in act_fsort */
 	switch (sort_mode) {
@@ -1918,11 +2205,10 @@ static void rescan_fnames()
  *  -------------------------------------------------
  */
 
-static int      compare_fnames(a,b)
 #ifdef ANSI_C
-const int   *a;
-const int   *b;
+static int      compare_fnames(const int *a,const int *b)
 #else
+static int      compare_fnames(a,b)
 int   *a;
 int   *b;
 #endif
@@ -1936,11 +2222,10 @@ int   *b;
  *  ------------------------------------------------
  */
 
-static int      compare_finfos(a,b)
 #ifdef ANSI_C
-const int   *a;
-const int   *b;
+static int      compare_finfos(const int *a,const int *b)
 #else
+static int      compare_finfos(a,b)
 int   *a;
 int   *b;
 #endif
