@@ -18,11 +18,11 @@
 /*--------------------------------------------------------------------*/
 
 #ifndef lint
-static char *id_string="$Id: alloc.c,v 3.7 1994/08/03 13:58:44 sander Exp $";
+static char *id_string="$Id: alloc.c,v 3.9 1995/02/08 11:11:14 sander Exp $";
 #endif
 
 /*
- *   Copyright (C) 1993, 1994 by Georg Sander, Iris Lemke, and
+ *   Copyright (C) 1993--1995 by Georg Sander, Iris Lemke, and
  *                               the Compare Consortium 
  *
  *  This program and documentation is free software; you can redistribute 
@@ -46,6 +46,13 @@ static char *id_string="$Id: alloc.c,v 3.7 1994/08/03 13:58:44 sander Exp $";
 
 /* 
  * $Log: alloc.c,v $
+ * Revision 3.9  1995/02/08  11:11:14  sander
+ * Distribution version 1.3.
+ *
+ * Revision 3.8  1994/12/23  18:12:45  sander
+ * Manhatten layout added.
+ * Option interface cleared.
+ *
  * Revision 3.7  1994/08/03  13:58:44  sander
  * Horizontal order mechanism changed.
  * Attribute horizontal_order for edges added.
@@ -113,6 +120,16 @@ static char *id_string="$Id: alloc.c,v 3.7 1994/08/03 13:58:44 sander Exp $";
  *			other edge. See the nearedge-specification-feature.
  *			We use a special adjacency list to notify all 
  *			near edges.
+ * bentnear_edge_insert insert a bent near edge into bent_near_edge_list. 
+ *			A bent near edge is a special edge consisting of a
+ *			near edge, a dummy node and a normal edge.
+ *			See the nearedge-specification-feature.
+ *			We use a special adjacency list to notify all 
+ *			bent near edges.
+ * back_edge_insert     insert a back edge into back_edge_list. A back edge
+ *			is an edge that preferably is reverted.
+ *			We use a special adjacency list to notify all 
+ *			back edges.
  * prededgealloc	allocates an edge list element (i.e. a cons cell whose
  *                      head is a GEDGE object), used as adjacency list of
  *                      predecessors of a node.
@@ -137,6 +154,7 @@ static char *id_string="$Id: alloc.c,v 3.7 1994/08/03 13:58:44 sander Exp $";
 #include "globals.h"
 #include "grammar.h"
 #include "main.h"
+#include "options.h"
 #include "alloc.h"
 #include "folding.h"
 
@@ -175,7 +193,7 @@ static long act_alloc_edges = 0;
 static long act_alloc_nodes = 0;
 #endif
 
-static int node_refnum = 0;	/* reference counter for REFNUM	of nodes */
+static long node_refnum = 0L;	/* reference counter for REFNUM	of nodes */
 
 
 /*   Core Memory allocation
@@ -184,8 +202,12 @@ static int node_refnum = 0;	/* reference counter for REFNUM	of nodes */
  *   parser. 
  */
 
+#ifdef ANSI_C
+char *myalloc(int x)
+#else
 char *myalloc(x)
 int 	x;
+#endif
 {
 	debugmessage("myalloc. Nr. of Bytes:",my_itoa(x));
 #ifdef DEBUG
@@ -201,7 +223,11 @@ int 	x;
  *   deallocate the complete memory.
  */
 
+#ifdef ANSI_C
+void 	free_memory(void)
+#else
 void 	free_memory()
+#endif
 {
 	debugmessage("free_memory","");
 #ifdef DEBUG
@@ -211,7 +237,7 @@ void 	free_memory()
 #endif
 	FreeHash();
 	ParseFree();
-	node_refnum = 0;	
+	node_refnum = 0L;	
 	reinit_all_lists();
 }
 
@@ -261,7 +287,11 @@ static GNODE node_freelist = NULL;     /* list of free GNODE objects */
  *  We also set some default values.
  */
 
+#ifdef ANSI_C
+static GNODE internal_nodealloc(void)
+#else
 static GNODE internal_nodealloc()
+#endif
 {
 	GNODE   h;
 
@@ -287,10 +317,10 @@ static GNODE internal_nodealloc()
 	NLEVEL(h) 	= -1;
 	NSHAPE(h) 	= 0;
 	NHORDER(h) 	= -1;
-	NSX(h)        	= 0;	
-	NSY(h)        	= 0;
-	NX(h)        	= 0;	
-	NY(h)        	= 0;
+	NSX(h)        	= 0L;	
+	NSY(h)        	= 0L;
+	NX(h)        	= 0L;	
+	NY(h)        	= 0L;
 	NSGRAPH(h)	= NULL;
 	NROOT(h)	= NULL;
 	NREGREPL(h)	= NULL;
@@ -300,13 +330,13 @@ static GNODE internal_nodealloc()
 	NINVISIBLE(h)	= 0;
 	NTIEFE(h)    	= -1;
 	NPOS(h)      	= -1;   
-	NWEIGHTS(h)     = 0;
-	NWEIGHTP(h)	= 0;
+	NWEIGHTS(h)     = 0L;
+	NWEIGHTP(h)	= 0L;
 	NMARK(h)     	= 0; 
 	NREVERT(h)     	= 0; 
 	NANCHORNODE(h) 	= 0; 
 	h->bary 	= -1;
-	NDFS(h)      	= -1; 
+	NDFS(h)      	= -1L; 
 	NINDEG(h)    	= 0;
 	NOUTDEG(h)	= 0;
 	NVPTR(h)	= NULL;
@@ -330,50 +360,173 @@ static GNODE internal_nodealloc()
  *  --------------------
  *  and update the nodelist. Real nodes are such nodes that are specified
  *  as node: { .... }.
+ *  We inheret the attributes from the refnode.
  */
 
-GNODE nodealloc(label,textm,width,height,borderw,xpos,ypos,fold,color,
-		textc,borderc,shrink,stretch,info1,info2,info3,level,shape,horder)
-char 	*label;
-int	textm,width,height,borderw,xpos,ypos,fold,color,textc,borderc;
-int	shrink,stretch;
-char 	*info1,*info2,*info3;
-int 	level;
-int 	shape;
-int     horder;
+#ifdef ANSI_C
+GNODE nodealloc(GNODE refnode)
+#else
+GNODE nodealloc(refnode)
+GNODE refnode;
+#endif
 {
-	GNODE   h;
+	GNODE h;
 
 	debugmessage("nodealloc","");
 	h = internal_nodealloc();
-
-	NLABEL(h)	= label;
-	NLEVEL(h)	= level;
-	NSHAPE(h)	= (char)shape;
-	NHORDER(h)	= horder;
-	NINFO1(h)	= info1;
-	NINFO2(h)	= info2;
-	NINFO3(h)	= info3;
-	NSX(h)		= xpos;
-	NSY(h)		= ypos;
-	NTEXTMODE(h)	= textm;
-	NSTATE(h)    	= 0;   
-	NWIDTH(h)    	= width;   
-	NHEIGHT(h)   	= height;
-	NBORDERW(h)	= borderw;
-	NFOLDING(h)	= fold;
-	NCOLOR(h)	= color;
-	NTCOLOR(h)	= textc;
-	NBCOLOR(h)	= borderc;
-	NSHRINK(h)	= shrink;
-	NSTRETCH(h)	= stretch;
-	NBEFORE(h)	= nodelistend;
-
+	copy_nodeattributes(refnode, h);
+	NBEFORE(h)      = nodelistend;
 	if (nodelistend) NNEXT(nodelistend) = h;
 	nodelistend	= h;
 	nodeanz++;
 	if (nodelist == NULL) nodelist = h;
 	return(h);
+
+}
+
+/*  Initialize a node with the node defaults
+ *  ----------------------------------------
+ */
+ 
+#ifdef ANSI_C
+void nodedefaults(GNODE node)
+#else
+void nodedefaults(node)
+GNODE node;
+#endif
+{
+	debugmessage("nodedefaults","");
+
+	NTITLE(node)	= G_title;
+	NLABEL(node)	= NULL;
+	NLEVEL(node)	= -1;
+	NSHAPE(node)	= BOX;
+	NHORDER(node)	= -1;
+	NINFO1(node)	= "";
+	NINFO2(node)	= "";
+	NINFO3(node)	= "";
+	NSX(node)	= 0L;
+	NSY(node)	= 0L;
+	NTEXTMODE(node)	= CENTER;
+	NSTATE(node)    = 0;
+	NWIDTH(node)    = -1;
+	NHEIGHT(node)   = -1;
+	NBORDERW(node)	= 2;
+	NFOLDING(node)	= -1;
+	NCOLOR(node)	= G_color;
+	NTCOLOR(node)	= BLACK;
+	NBCOLOR(node)	= BLACK;
+	NSHRINK(node)	= 1;
+	NSTRETCH(node)	= 1;
+}
+
+
+/*  Initialize a node with the foldnode defaults
+ *  --------------------------------------------
+ */
+
+#ifdef ANSI_C
+void foldnodedefaults(GNODE node)
+#else
+void foldnodedefaults(node)
+GNODE node;
+#endif
+{
+	debugmessage("foldnodedefaults","");
+
+	NTITLE(node)	= NULL;
+	NLABEL(node)	= NULL;
+	NLEVEL(node)	= -1;
+	NSHAPE(node)	= -1;
+	NHORDER(node)	= -1;
+	NINFO1(node)	= NULL;
+	NINFO2(node)	= NULL;
+	NINFO3(node)	= NULL;
+	NSX(node)	= 0L;
+	NSY(node)	= 0L;
+	NTEXTMODE(node)	= -1;
+	NSTATE(node)    = -1;
+	NWIDTH(node)    = -1;
+	NHEIGHT(node)   = -1;
+	NBORDERW(node)	= 4;
+	NFOLDING(node)	= -1;
+	NCOLOR(node)	= -1;
+	NTCOLOR(node)	= -1;
+	NBCOLOR(node)	= -1;
+	NSHRINK(node)	= -1;
+	NSTRETCH(node)	= -1;
+}
+
+
+/*  Copy the foldnode attributes from fn to y
+ *  -----------------------------------------
+ */
+
+#ifdef ANSI_C
+void inherit_foldnode_attributes(GNODE fn, GNODE y)
+#else
+void inherit_foldnode_attributes(fn, y)
+GNODE fn, y;
+#endif
+{
+	debugmessage("inherit_foldnode_attributes","");
+
+	/* NTITLE not needed */
+	/* NFOLDING not needed */
+	if (NLABEL(fn))	 	NLABEL(y)	= NLABEL(fn);
+	if (NLEVEL(fn)!= -1) 	NLEVEL(y)	= NLEVEL(fn);
+	if (NSHAPE(fn)!= -1) 	NSHAPE(y)	= NSHAPE(fn);
+	if (NHORDER(fn)!= -1) 	NHORDER(y)	= NHORDER(fn);
+	if (NINFO1(fn))	 	NINFO1(y)	= NINFO1(fn);
+	if (NINFO2(fn))	 	NINFO2(y)	= NINFO2(fn);
+	if (NINFO3(fn))	 	NINFO3(y)	= NINFO3(fn);
+	if (NSX(fn)!= -1L)	NSX(y)		= NSX(fn);
+	if (NSY(fn)!= -1L) 	NSY(y)	 	= NSY(fn);
+	if (NTEXTMODE(fn)!= -1)	NTEXTMODE(y)	= NTEXTMODE(fn);
+	if (NSTATE(fn)!= -1)  	NSTATE(y)     	= NSTATE(fn);
+	if (NWIDTH(fn)!= -1)  	NWIDTH(y)	= NWIDTH(fn);
+	if (NHEIGHT(fn)!= -1) 	NHEIGHT(y)    	= NHEIGHT(fn);
+	if (NBORDERW(fn)!= -1)	NBORDERW(y)	= NBORDERW(fn);
+	if (NCOLOR(fn)!= -1) 	NCOLOR(y)	= NCOLOR(fn);
+	if (NTCOLOR(fn)!= -1)	NTCOLOR(y)	= NTCOLOR(fn);
+	if (NBCOLOR(fn)!= -1) 	NBCOLOR(y)	= NBCOLOR(fn);
+	if (NSHRINK(fn)!= -1) 	NSHRINK(y)	= NSHRINK(fn);
+	if (NSTRETCH(fn)!= -1) 	NSTRETCH(y)	= NSTRETCH(fn);
+}
+
+
+/*  Copy a GNODE object x into a second GNODE object y
+ *  --------------------------------------------------
+ */
+
+#ifdef ANSI_C
+void copy_nodeattributes(GNODE x, GNODE y)
+#else
+void copy_nodeattributes(x, y)
+GNODE x, y;
+#endif
+{
+	NTITLE(y)	= NTITLE(x);
+	NLABEL(y)	= NLABEL(x);
+	NLEVEL(y)	= NLEVEL(x);
+	NSHAPE(y)	= NSHAPE(x);
+	NHORDER(y)	= NHORDER(x);
+	NINFO1(y)	= NINFO1(x);
+	NINFO2(y)	= NINFO2(x);
+	NINFO3(y)	= NINFO3(x);
+	NSX(y)		= NSX(x);
+	NSY(y)		= NSY(x);
+	NTEXTMODE(y)	= NTEXTMODE(x);
+	NSTATE(y)    	= NSTATE(x);
+	NWIDTH(y)    	= NWIDTH(x);
+	NHEIGHT(y)   	= NHEIGHT(x);
+	NBORDERW(y)	= NBORDERW(x);
+	NFOLDING(y)	= NFOLDING(x);
+	NCOLOR(y)	= NCOLOR(x);
+	NTCOLOR(y)	= NTCOLOR(x);
+	NBCOLOR(y)	= NBCOLOR(x);
+	NSHRINK(y)	= NSHRINK(x);
+	NSTRETCH(y)	= NSTRETCH(x);
 }
 
 
@@ -382,46 +535,25 @@ int     horder;
  *  -----------------------
  *  and update the graphlist. Such objects are summary nodes whose graph
  *  is specified as graph: { .... }. 
+ *  We inheret the attributes from the refnode.
  */
 
-GNODE graphalloc(label,textm,width,height,borderw,xpos,ypos,fold,color,
-		 textc,borderc,shrink,stretch,info1,info2,info3,level,shape,horder)
-char 	*label;
-int	textm,width,height,borderw,xpos,ypos,fold,color,textc,borderc;
-int	shrink,stretch;
-char 	*info1,*info2,*info3;
-int	level;
-int 	shape;
-int 	horder;
+#ifdef ANSI_C
+GNODE graphalloc(GNODE refnode)
+#else
+GNODE graphalloc(refnode)
+GNODE refnode;
+#endif
 {
 	GNODE   h;
 
 	debugmessage("graphalloc","");
 	h = internal_nodealloc();
-
-	NLABEL(h)	= label;
-	NLEVEL(h)	= level;
-	NSHAPE(h)	= (char)shape;
-	NHORDER(h)	= horder;
-	NINFO1(h)	= info1;
-	NINFO2(h)	= info2;
-	NINFO3(h)	= info3;
-	NSX(h)		= xpos;
-	NSY(h)		= ypos;
-	NTEXTMODE(h)	= textm;
-	NSTATE(h)    	= 0;   
-	NWIDTH(h)	= width;
-	NHEIGHT(h)	= height;
-	NBORDERW(h)	= borderw;
-	NFOLDING(h)	= fold; 
-	NCOLOR(h)	= color;
-	NTCOLOR(h)	= textc;
-	NBCOLOR(h)	= borderc;
-	NSHRINK(h)	= shrink;
-	NSTRETCH(h)	= stretch;	
+	copy_nodeattributes(refnode, h);
+	NFOLDING(h)	= -1;
 	NINLIST(h)	= 0;
 	NINVISIBLE(h) 	= 1;
-	NDFS(h)		= 0;
+	NDFS(h)		= 0L;
 	NBEFORE(h)	= graphlistend;
 
 	if (graphlistend) NNEXT(graphlistend) = h;
@@ -437,10 +569,25 @@ int 	horder;
  *  These nodes are only needed for the layouting.
  */
 
+#ifdef ANSI_C
+GNODE	tmpnodealloc(
+	int     textm,
+	int	width,
+	int	height,
+	int	borderw,
+	int	fold,
+	int	color,
+	int	textc,
+	int	borderc,
+	int     shrink,
+	int	stretch,
+	int	horder)
+#else
 GNODE	tmpnodealloc(textm,width,height,borderw,fold,color,textc,borderc,
                  	    shrink,stretch,horder)
 int     textm,width,height,borderw,fold,color,textc,borderc;
 int     shrink,stretch,horder;
+#endif
 {
 	GNODE	h;
 
@@ -461,7 +608,7 @@ int     shrink,stretch,horder;
         NSTRETCH(h)     = stretch;
         NINLIST(h)      = 0;
         NINVISIBLE(h)   = 1;
-        NDFS(h)         = 0;
+        NDFS(h)         = 0L;
         NBEFORE(h)      = NULL;
 	
 	NINTERN(h)   = tmpnodelist;
@@ -474,7 +621,11 @@ int     shrink,stretch,horder;
  *  --------------------------------------
  */
 
+#ifdef ANSI_C
+void free_tmpnodes(void)
+#else
 void free_tmpnodes()
+#endif
 {
 	GNODE	h;
 
@@ -503,8 +654,12 @@ void free_tmpnodes()
  *  it could be given free twice, which is wrong.
  */
 
+#ifdef ANSI_C
+void free_node(GNODE h)
+#else
 void free_node(h)
 GNODE h;
+#endif
 {
 	debugmessage("free_node","");
 	NINTERN(h) = node_freelist;
@@ -518,11 +673,16 @@ GNODE h;
  *  At this time point, all visible nodes are in the node list.
  */
 
+#ifdef ANSI_C
+GNODE	search_xy_node(long x,long y)
+#else
 GNODE	search_xy_node(x,y)
-int	x,y;
+long	x,y;
+#endif
 {
 	GNODE	v;
-	int	xpos, ypos, width, height;
+	int	width, height;
+	long	xpos, ypos;
 
 	v = nodelist;
 	while (v) {
@@ -546,7 +706,11 @@ int	x,y;
  *  Thus, for such subgraphs, we add auxiliary nodes.
  */
 
+#ifdef ANSI_C
+void check_graph_consistency(void)
+#else
 void check_graph_consistency()
+#endif
 {
 	GNODE v,w;
 
@@ -561,25 +725,7 @@ void check_graph_consistency()
 				FPRINTF(stderr," I add a node ! \n");
 			}
 			
-			w = nodealloc(  NLABEL(v),
-                        		NTEXTMODE(v),
-                        		NWIDTH(v),
-                        		NHEIGHT(v),
-                        		NBORDERW(v),
-                        		NSX(v),
-                        		NSY(v),
-                        		NFOLDING(v),
-                        		NCOLOR(v),
-                        		NTCOLOR(v),
-                        		NBCOLOR(v),
-                        		NSHRINK(v),
-                        		NSTRETCH(v),
-                        		NINFO1(v),
-                        		NINFO2(v),
-                        		NINFO3(v),
-                        		NLEVEL(v),
-                        		NSHAPE(v),
-					NHORDER(v));
+			w = nodealloc( v );
 			NTITLE(w) = "artificial node";
 			NROOT(w) = v;
 			NSGRAPH(v) = nodelist_alloc(w);
@@ -617,8 +763,12 @@ static GNLIST ncons_freelist = NULL;  /* list of free cons cells      */
  *  store them in the tmpnconslist.
  */
 
+#ifdef ANSI_C
+GNLIST  nodelist_alloc(GNODE v)
+#else
 GNLIST  nodelist_alloc(v)
 GNODE v;
+#endif
 {
 	GNLIST	h;
 
@@ -639,7 +789,11 @@ GNODE v;
  *  tmpnconslist, to give them free later.
  */
 
+#ifdef ANSI_C
+GNLIST  tmpnodelist_alloc(void)
+#else
 GNLIST  tmpnodelist_alloc()
+#endif
 {
 	GNLIST	h;
 
@@ -667,7 +821,11 @@ GNLIST  tmpnodelist_alloc()
  *  also temporary, because they are deallocated after folding.
  */
 
+#ifdef ANSI_C
+GNLIST  foldnodelist_alloc(void)
+#else
 GNLIST  foldnodelist_alloc()
+#endif
 {
 	GNLIST	h;
 
@@ -690,7 +848,11 @@ GNLIST  foldnodelist_alloc()
  *  --------------------------------------
  */
 
+#ifdef ANSI_C
+static void free_nodelists(void)
+#else
 static void free_nodelists()
+#endif
 {
 	GNLIST	h;
 
@@ -709,7 +871,11 @@ static void free_nodelists()
  *  ----------------------------------
  */
 
+#ifdef ANSI_C
+void free_foldnodelists(void)
+#else
 void free_foldnodelists()
+#endif
 {
 	GNLIST	h;
 
@@ -730,8 +896,12 @@ void free_foldnodelists()
  *  i.e. should not be temporary.
  */
 
+#ifdef ANSI_C
+void free_regionnodelist(GNLIST	r)
+#else
 void free_regionnodelist(r)
 GNLIST	r;
+#endif
 {
 	GNLIST	h;
 
@@ -782,7 +952,11 @@ static GEDGE edge_freelist = NULL;     /* list of free GEDGE objects        */
  *  We also set some default values.
  */
 
+#ifdef ANSI_C
+static GEDGE internal_edgealloc(void)
+#else
 static GEDGE internal_edgealloc()
+#endif
 {
 	GEDGE   h;
 
@@ -813,12 +987,14 @@ static GEDGE internal_edgealloc()
 	EORI2(h)	= NO_ORI;
 	ELABEL(h)  	= NULL;
 	EART(h)       	= 'U';
+	ELNODE(h)      	= NULL;
 	EANCHOR(h)	= 0;
 	EINVISIBLE(h)	= 0;
 	EWEIGHTS(h)   	= 0;
 	EWEIGHTP(h)   	= 0;
 	ENEXT(h)	= NULL;
 	EINTERN(h)	= NULL;
+	ELABELCOL(h)	= BLACK;
 	return(h);
 }
 
@@ -827,27 +1003,22 @@ static GEDGE internal_edgealloc()
  *  --------------------
  *  and update the edgelist. These edges are specified, e.g.
  *  as edge: { ... }
+ *  We inheret the attributes from the refedge.
  */
 
-GEDGE edgealloc(label,lstyle,thick,class,prio,ecolor,arrows,horder)
-char 	*label;
-int	lstyle,thick,class,prio,ecolor,arrows,horder;
+#ifdef ANSI_C
+GEDGE edgealloc(GEDGE refedge)
+#else
+GEDGE edgealloc(refedge)
+GEDGE refedge;
+#endif
 {
 	GEDGE   h;
  
 	debugmessage("edgealloc","");
 	h = internal_edgealloc();
-
-	ELABEL(h)	= label;
-	ELSTYLE(h)    	= lstyle;
-	ETHICKNESS(h)	= thick;
-	ECLASS(h)	= class;
-	EPRIO(h)	= prio;
-	EHORDER(h)	= horder;
-	ECOLOR(h)	= ecolor;
-	EARROWSIZE(h)	= arrows;
+	copy_edgeattributes(refedge, h);
 	EBEFORE(h)	= edgelistend;
-
 	if (edgelistend) ENEXT(edgelistend) = h;
 	edgelistend     = h;
 	edgeanz++;
@@ -856,6 +1027,123 @@ int	lstyle,thick,class,prio,ecolor,arrows,horder;
 }
 
 
+/*  Initialize an edge with the edge defaults
+ *  -----------------------------------------
+ */
+ 
+#ifdef ANSI_C
+void edgedefaults(GEDGE edge)
+#else
+void edgedefaults(edge)
+GEDGE edge;
+#endif
+{
+	debugmessage("edgedefaults","");
+
+	ELABEL(edge)		= NULL;
+	ELSTYLE(edge)    	= SOLID;
+	ETHICKNESS(edge)	= 2;
+	ECLASS(edge)		= 1;
+	EPRIO(edge)		= 1;
+	EHORDER(edge)		= -1;
+	ECOLOR(edge)		= BLACK;
+	ELABELCOL(edge)		= BLACK;
+	EARROWSIZE(edge)	= 10;
+	EARROWSTYLE(edge)	= ASSOLID;
+	EARROWCOL(edge)		= BLACK;
+	EARROWBSIZE(edge)	= 0;
+	EARROWBSTYLE(edge)	= ASNONESPEC;
+	EARROWBCOL(edge)	= BLACK;
+}
+
+
+/*  Initialize an edge with the foldedge defaults
+ *  ---------------------------------------------
+ */
+
+#ifdef ANSI_C
+void foldedgedefaults(GEDGE edge)
+#else
+void foldedgedefaults(edge)
+GEDGE edge;
+#endif
+{
+	debugmessage("foldedgedefaults","");
+
+	ELABEL(edge)		= "...";
+	ELSTYLE(edge)    	= -1;
+	ETHICKNESS(edge)	= 4;
+	ECLASS(edge)		= -1;
+	EPRIO(edge)		= -1;
+	EHORDER(edge)		= -1;
+	ECOLOR(edge)		= -1;
+	ELABELCOL(edge)		= -1;
+	EARROWSIZE(edge)	= -1;
+	EARROWSTYLE(edge)	= -1;
+	EARROWCOL(edge)		= -1;
+	EARROWBSIZE(edge)	= -1;
+	EARROWBSTYLE(edge)	= -1;
+	EARROWBCOL(edge)	= -1;
+}
+
+
+/*  Copy the foldedge attributes from fn to y
+ *  -----------------------------------------
+ */
+
+#ifdef ANSI_C
+void inherit_foldedge_attributes(GEDGE fn, GEDGE y)
+#else
+void inherit_foldedge_attributes(fn, y)
+GEDGE fn, y;
+#endif
+{
+	debugmessage("inherit_foldedge_attributes","");
+
+	if (ELABEL(fn)) 	   ELABEL(y)       = ELABEL(fn);
+	if (ELSTYLE(fn)     != -1) ELSTYLE(y)      = ELSTYLE(fn);
+	if (ETHICKNESS(fn)  != -1) ETHICKNESS(y)   = ETHICKNESS(fn);
+	if (ECLASS(fn)      != -1) ECLASS(y)       = ECLASS(fn);
+	if (EPRIO(fn)       != -1) EPRIO(y)        = EPRIO(fn);
+	if (EHORDER(fn)     != -1) EHORDER(y)      = EHORDER(fn);
+	if (ECOLOR(fn)      != -1) ECOLOR(y)       = ECOLOR(fn);
+	if (ELABELCOL(fn)   != -1) ELABELCOL(y)    = ELABELCOL(fn);
+	if (EARROWSIZE(fn)  != -1) EARROWSIZE(y)   = EARROWSIZE(fn);
+	if (EARROWSTYLE(fn) != -1) EARROWSTYLE(y)  = EARROWSTYLE(fn);
+	if (EARROWCOL(fn)   != -1) EARROWCOL(y)    = EARROWCOL(fn);
+	if (EARROWBSIZE(fn) != -1) EARROWBSIZE(y)  = EARROWBSIZE(fn);
+	if (EARROWBSTYLE(fn)!= -1) EARROWBSTYLE(y) = EARROWBSTYLE(fn);
+	if (EARROWBCOL(fn)  != -1) EARROWBCOL(y)   = EARROWBCOL(fn);
+}
+
+
+/*  Copy a GEDGE object x into a second GEDGE object y
+ *  --------------------------------------------------
+ */
+
+#ifdef ANSI_C
+void copy_edgeattributes(GEDGE x, GEDGE y)
+#else
+void copy_edgeattributes(x, y)
+GEDGE x, y;
+#endif
+{
+	ELABEL(y)	= ELABEL(x);
+	ELSTYLE(y)    	= ELSTYLE(x);
+	ETHICKNESS(y)	= ETHICKNESS(x);
+	ECLASS(y)	= ECLASS(x);
+	EPRIO(y)	= EPRIO(x);
+	EHORDER(y)	= EHORDER(x);
+	ECOLOR(y)	= ECOLOR(x);
+	ELABELCOL(y)	= ELABELCOL(x);
+	EARROWSIZE(y)	= EARROWSIZE(x);
+	EARROWSTYLE(y)	= EARROWSTYLE(x);
+	EARROWCOL(y)	= EARROWCOL(x);
+	EARROWBSIZE(y)	= EARROWBSIZE(x);
+	EARROWBSTYLE(y)	= EARROWBSTYLE(x);
+	EARROWBCOL(y)	= EARROWBCOL(x);
+}
+
 
 
 /*  Allocate a temporary GEDGE object 
@@ -864,9 +1152,27 @@ int	lstyle,thick,class,prio,ecolor,arrows,horder;
  *  to labels.
  */
 
-
-GEDGE	tmpedgealloc(lstyle,thick,class,prio,ecolor,arrows,horder)
-int	lstyle,thick,class,prio,ecolor,arrows,horder;
+#ifdef ANSI_C
+GEDGE	tmpedgealloc(
+	int	lstyle,
+	int	thick,
+	int	xclass,
+	int	prio,
+	int	ecolor,
+	int	elcol,
+	int	arrows,
+	int	barrows,
+	int	arrowsty,
+	int	barrowsty,
+	int	arrowc,
+	int	barrowc,
+	int	horder)
+#else
+GEDGE	tmpedgealloc(lstyle,thick,xclass,prio,ecolor,elcol,arrows,
+		barrows,arrowsty,barrowsty,arrowc,barrowc,horder)
+int	lstyle,thick,xclass,prio,ecolor,elcol,arrows,
+	barrows,arrowsty,barrowsty,arrowc,barrowc,horder;
+#endif
 {
 	GEDGE	h;
 
@@ -875,12 +1181,20 @@ int	lstyle,thick,class,prio,ecolor,arrows,horder;
 
 	ELSTYLE(h)    	= lstyle;
 	ETHICKNESS(h)	= thick;
-	ECLASS(h)	= class;
+	ECLASS(h)	= xclass;
 	EPRIO(h)	= prio;
 	EHORDER(h)	= horder;
 	ECOLOR(h)	= ecolor;
-	EARROWSIZE(h)	= arrows;
+	ELABELCOL(h)	= elcol;
 	EBEFORE(h)	= NULL;
+	EARROWSTYLE(h)	= ASSOLID;
+	EARROWCOL(h)	= ecolor;
+	EARROWSIZE(h)	= arrows;
+	EARROWSTYLE(h)	= arrowsty;
+	EARROWCOL(h)	= arrowc;
+	EARROWBSIZE(h)	= barrows;
+	EARROWBSTYLE(h)	= barrowsty;
+	EARROWBCOL(h)	= barrowc;
 
 	EINTERN(h) = tmpedgelist;
 	tmpedgelist = h;
@@ -892,7 +1206,11 @@ int	lstyle,thick,class,prio,ecolor,arrows,horder;
  *  --------------------------------------
  */
 
+#ifdef ANSI_C
+static void free_tmpedges(void)
+#else
 static void free_tmpedges()
+#endif
 {
 	GEDGE	h;
 
@@ -921,12 +1239,16 @@ static void free_tmpedges()
  *  Further, we have one nontemporary list of edges that contains the
  *  default connections as specified by `near_edge'. This is the
  *  near_edge_list.
+ *  Dito, we have one nontemporary list of edges that contains the
+ *  edges specified by `back_edge'. This is the back_edge_list.
  */
 
 
 /* for stable default connections: */
 
-ADJEDGE near_edge_list = NULL;	/* list of default connections */
+ADJEDGE near_edge_list = NULL;	    /* list of default connections */
+ADJEDGE bent_near_edge_list = NULL; /* list of bent near edges     */
+ADJEDGE back_edge_list = NULL;	    /* list of back edges          */
 
 /* for temporaries: */
 
@@ -940,8 +1262,12 @@ static ADJEDGE econs_freelist = NULL;  /* list of free cons cells      */
  *  we allocate a cell from the core memory.
  */
 
+#ifdef ANSI_C
+void near_edge_insert(GEDGE e)
+#else
 void near_edge_insert(e)
 GEDGE e;
+#endif
 {
 	ADJEDGE	h;
 
@@ -957,13 +1283,73 @@ GEDGE e;
 }
 
 
+/*  Insert a bent near edge into bent_near_edge_list
+ *  ------------------------------------------------
+ *  First, we look in the free list, if we have a free cell. Otherwise,
+ *  we allocate a cell from the core memory.
+ */
+
+#ifdef ANSI_C
+void bentnear_edge_insert(GEDGE e)
+#else
+void bentnear_edge_insert(e)
+GEDGE e;
+#endif
+{
+	ADJEDGE	h;
+
+	debugmessage("bentnear_edge_insert","");
+	if (econs_freelist) {
+		h = econs_freelist;
+		econs_freelist = AINTERN(econs_freelist);
+	}
+	else    h = (ADJEDGE)myalloc(sizeof(struct adjedge));
+	AKANTE(h) = e;
+	ANEXT(h) = AINTERN(h) = bent_near_edge_list;
+	bent_near_edge_list = h;
+}
+
+
+
+/*  Insert a back edge into back_edge_list
+ *  --------------------------------------
+ *  First, we look in the free list, if we have a free cell. Otherwise,
+ *  we allocate a cell from the core memory.
+ */
+
+#ifdef ANSI_C
+void back_edge_insert(GEDGE e)
+#else
+void back_edge_insert(e)
+GEDGE e;
+#endif
+{
+	ADJEDGE	h;
+
+	debugmessage("back_edge_insert","");
+	if (econs_freelist) {
+		h = econs_freelist;
+		econs_freelist = AINTERN(econs_freelist);
+	}
+	else    h = (ADJEDGE)myalloc(sizeof(struct adjedge));
+	AKANTE(h) = e;
+	ANEXT(h) = AINTERN(h) = back_edge_list;
+	back_edge_list = h;
+}
+
+
+
 /*  Allocate a ADJEDGE object
  *  -------------------------
  *  First, we look in the free list, if we have a free cell. Otherwise,
  *  we allocate a cell from the core memory.
  */
 
+#ifdef ANSI_C
+static ADJEDGE  edgelist_alloc(void)
+#else
 static ADJEDGE  edgelist_alloc()
+#endif
 {
 	ADJEDGE	h;
 
@@ -983,9 +1369,13 @@ static ADJEDGE  edgelist_alloc()
  *  --------------------------------------------
  */
 
+#ifdef ANSI_C
+ADJEDGE prededgealloc(GNODE node, GEDGE edge)
+#else
 ADJEDGE prededgealloc(node,edge)
 GNODE	node;	
 GEDGE   edge;
+#endif
 {
 	ADJEDGE e;
         
@@ -1002,9 +1392,13 @@ GEDGE   edge;
  *  -----------------------------------------------
  */
 
+#ifdef ANSI_C
+ADJEDGE succedgealloc(GNODE node, GEDGE edge)
+#else
 ADJEDGE succedgealloc(node,edge)
 GNODE   node;        
 GEDGE   edge;
+#endif
 {
 	ADJEDGE e;   
         
@@ -1021,7 +1415,11 @@ GEDGE   edge;
  *  ----------------------------------------
  */
 
+#ifdef ANSI_C
+static void free_edgelists(void)
+#else
 static void free_edgelists()
+#endif
 {
 	ADJEDGE	h;
 
@@ -1063,8 +1461,12 @@ static CONNECT 	connect_freelist = NULL;   /* list of free   connect cells */
  *  GNODE node.
  */
 
+#ifdef ANSI_C
+CONNECT	connectalloc(GNODE node)
+#else
 CONNECT	connectalloc(node)
 GNODE	node;
+#endif
 {
 	CONNECT	h;
 
@@ -1089,7 +1491,11 @@ GNODE	node;
  *  ----------------------------------------
  */
 
+#ifdef ANSI_C
+static void free_connect(void)
+#else
 static void free_connect()
+#endif
 {
 	CONNECT	h;
 		
@@ -1132,9 +1538,13 @@ static DLLIST	dllist_freelist  = NULL;     /* list of free dllist nodes */
  */
 
 
+#ifdef ANSI_C
+DLLIST 	dllist_alloc(GNODE node, DLLIST pred)
+#else
 DLLIST 	dllist_alloc(node,pred)
 GNODE  node;
 DLLIST pred;
+#endif
 {
 	DLLIST	h;
 
@@ -1158,8 +1568,12 @@ DLLIST pred;
  *  Thus we don't need any additional memory management.
  */
 
+#ifdef ANSI_C
+void	dllist_free(DLLIST x)
+#else
 void	dllist_free(x)
 DLLIST x;
+#endif
 {
 	debugmessage("dllist_free","");
        	DSUCC(x) = dllist_freelist;
@@ -1171,8 +1585,12 @@ DLLIST x;
  *  -----------------------------------
  */
 
+#ifdef ANSI_C
+void	dllist_free_all(DLLIST x)
+#else
 void	dllist_free_all(x)
 DLLIST x;
+#endif
 {
 	DLLIST h;
 
@@ -1195,7 +1613,11 @@ DLLIST x;
  */
 
 
+#ifdef ANSI_C
+void	free_all_lists(void)
+#else
 void	free_all_lists()
+#endif
 {
 	free_tmpnodes();
 	free_tmpedges();
@@ -1209,7 +1631,11 @@ void	free_all_lists()
  *  --------------------------------------------
  */
 
+#ifdef ANSI_C
+void    reinit_all_lists(void)
+#else
 void    reinit_all_lists()
+#endif
 {
 	ufoldstart  = NULL;
         foldstart   = NULL;
@@ -1239,9 +1665,11 @@ void    reinit_all_lists()
 	tmpedgelist   	 = NULL;
 	edge_freelist    = NULL;
 
-	near_edge_list   = NULL;
-	tmpeconslist     = NULL;
-	econs_freelist   = NULL;
+	near_edge_list      = NULL;
+	back_edge_list      = NULL;
+	bent_near_edge_list = NULL;
+	tmpeconslist        = NULL;
+	econs_freelist      = NULL;
 
 	connectlist	 = NULL;
 	connect_freelist = NULL;
