@@ -18,12 +18,12 @@
 
 
 #ifndef lint
-static char *id_string="$Id: grprint.c,v 3.6 1994/08/03 13:58:44 sander Exp $";
+static char *id_string="$Id: grprint.c,v 3.8 1995/02/08 11:11:14 sander Exp $";
 #endif
 
 
 /*
- *   Copyright (C) 1993, 1994 by Georg Sander, Iris Lemke, and
+ *   Copyright (C) 1993--1995 by Georg Sander, Iris Lemke, and
  *                               the Compare Consortium 
  *
  *  This program and documentation is free software; you can redistribute 
@@ -47,6 +47,16 @@ static char *id_string="$Id: grprint.c,v 3.6 1994/08/03 13:58:44 sander Exp $";
 
 /* 
  * $Log: grprint.c,v $
+ * Revision 3.8  1995/02/08  11:11:14  sander
+ * Distribution version 1.3.
+ *
+ * Revision 3.7  1994/12/23  18:12:45  sander
+ * Manhatten layout added.
+ * Option interface cleared.
+ * infobox behaviour improved.
+ * First version of fisheye (carthesian).
+ * Options Noedge and nonode.
+ *
  * Revision 3.6  1994/08/03  13:58:44  sander
  * Horizontal order mechanism changed.
  * Attribute horizontal_order for edges added.
@@ -98,8 +108,11 @@ static char *id_string="$Id: grprint.c,v 3.6 1994/08/03 13:58:44 sander Exp $";
 #include <stdlib.h>
 #include "globals.h"
 #include "main.h"
+#include "options.h"
 #include "alloc.h"
 #include "folding.h"
+#include "steps.h"
+#include "fisheye.h"
 #include "grprint.h"
 
 /*  Prototypes
@@ -132,8 +145,12 @@ static int actindent;
 #define backward_connection2(c) ((CEDGE2(c))&&(EEND(CEDGE2(c))==v))
 
 
+#ifdef ANSI_C
+int print_graph(char *fname)
+#else
 int print_graph(fname)
 char *fname;
+#endif
 {
 	GNODE v;
 	ADJEDGE li;
@@ -221,8 +238,12 @@ char *fname;
 /*   Open a graph 			         		      */
 /*--------------------------------------------------------------------*/
 
+#ifdef ANSI_C
+static void open_graph(FILE *f)
+#else
 static void open_graph(f)
 FILE *f;
+#endif
 {
 	int k;
 
@@ -234,19 +255,32 @@ FILE *f;
 	INDPRINT(f,"height: %d\n",   G_height);
 	INDPRINT(f,"xmax: %d\n",     G_xmax);
 	INDPRINT(f,"ymax: %d\n",     G_ymax);
-	INDPRINT(f,"xbase: %d\n",    G_xbase);
-	INDPRINT(f,"ybase: %d\n",    G_ybase);
+	INDPRINT(f,"xbase: %ld\n",   G_xbase);
+	INDPRINT(f,"ybase: %ld\n",   G_ybase);
 	INDPRINT(f,"xspace: %d\n",   G_xspace);
 	INDPRINT(f,"xlspace: %d\n",  G_dspace);
 	INDPRINT(f,"yspace: %d\n",   G_yspace);
-	INDPRINT(f,"x: %d \n",  G_x );
-	INDPRINT(f,"y: %d\n",   G_y);
+	INDPRINT(f,"xraster: %d\n",  G_xraster);
+	INDPRINT(f,"xlraster: %d\n", G_dxraster);
+	INDPRINT(f,"yraster: %d\n",  G_yraster);
+	INDPRINT(f,"x: %ld \n", G_x );
+	INDPRINT(f,"y: %ld\n",  G_y);
 	INDPRINT(f,"shrink:  %d\n",  G_shrink);
 	INDPRINT(f,"stretch: %d\n",  G_stretch);
 	INDPRINT(f,"layout_downfactor: %d\n",  layout_downfactor);
 	INDPRINT(f,"layout_upfactor: %d\n",    layout_upfactor);
 	INDPRINT(f,"layout_nearfactor: %d\n",  layout_nearfactor);
 	INDPRINT(f,"layout_splinefactor: %d\n",G_flat_factor);
+	INDPRINT(f,"spreadlevel: %d\n",spread_level);
+	INDPRINT(f,"treefactor: %f\n", tree_factor);
+	INDPRINT(f,"bmax: %d\n",max_edgebendings);
+	INDPRINT(f,"cmin: %d\n",min_baryiterations);
+	INDPRINT(f,"cmax: %d\n",max_baryiterations);
+	INDPRINT(f,"pmin: %d\n",min_mediumshifts);
+	INDPRINT(f,"pmax: %d\n",max_mediumshifts);
+	INDPRINT(f,"rmin: %d\n",min_centershifts);
+	INDPRINT(f,"rmax: %d\n",max_centershifts);
+	INDPRINT(f,"smax: %d\n",max_straighttune);
 
 	switch (G_yalign) {
 	case AL_TOP:	
@@ -267,28 +301,76 @@ FILE *f;
 		INDPRINT(f,"orientation: right_to_left%s\n",""); break;
 	}
 	switch (edge_label_phase) {
-	case 0: INDPRINT(f,"late_edge_labels: no%s\n",""); break;
 	case 1: INDPRINT(f,"late_edge_labels: yes%s\n",""); break;
+	default:INDPRINT(f,"late_edge_labels: no%s\n",""); break;
 	}
 	switch (G_displayel) {
-	case 0: INDPRINT(f,"display_edge_labels: no%s\n",""); break;
 	case 1: INDPRINT(f,"display_edge_labels: yes%s\n",""); break;
+	default:INDPRINT(f,"display_edge_labels: no%s\n",""); break;
 	}
 	switch (G_dirtyel) {
-	case 0: INDPRINT(f,"dirty_edge_labels: no%s\n",""); break;
 	case 1: INDPRINT(f,"dirty_edge_labels: yes%s\n",""); break;
+	default:INDPRINT(f,"dirty_edge_labels: no%s\n",""); break;
 	}
 	switch (fine_tune_layout) {
-	case 0: INDPRINT(f,"finetuning: no%s\n",""); break;
 	case 1: INDPRINT(f,"finetuning: yes%s\n",""); break;
+	default:INDPRINT(f,"finetuning: no%s\n",""); break;
 	}
 	switch (near_edge_layout) {
-	case 0: INDPRINT(f,"nearedges: no%s\n",""); break;
 	case 1: INDPRINT(f,"nearedges: yes%s\n",""); break;
+	default:INDPRINT(f,"nearedges: no%s\n",""); break;
 	}
 	switch (G_spline) {
-	case 0: INDPRINT(f,"splines: no%s\n",""); break;
 	case 1: INDPRINT(f,"splines: yes%s\n",""); break;
+	default:INDPRINT(f,"splines: no%s\n",""); break;
+	}
+	switch (hide_single_nodes) {
+	case 1: INDPRINT(f,"ignoresingles: yes%s\n",""); break;
+	default:INDPRINT(f,"ignoresingles: no%s\n",""); break;
+	}
+	switch (straight_phase) {
+	case 1:  INDPRINT(f,"straight_phase: yes%s\n",""); break;
+	default: INDPRINT(f,"straight_phase: no%s\n",""); break;
+	}
+	switch (prio_phase) {
+	case 1:  INDPRINT(f,"priority_phase: yes%s\n",""); break;
+	default: INDPRINT(f,"priority_phase: no%s\n",""); break;
+	}
+	switch (manhatten_edges) {
+	case 1:  INDPRINT(f,"manhatten_edges: yes%s\n",""); break;
+	default: INDPRINT(f,"manhatten_edges: no%s\n",""); break;
+	}
+	switch (one_line_manhatten) {
+	case 1:  INDPRINT(f,"smanhatten_edges: yes%s\n",""); break;
+	default: INDPRINT(f,"smanhatten_edges: no%s\n",""); break;
+	}
+	switch (G_portsharing) {
+	case 1:  INDPRINT(f,"port_sharing: yes%s\n",""); break;
+	default: INDPRINT(f,"port_sharing: no%s\n",""); break;
+	}
+	switch (skip_baryphase2) {
+	case 1:  INDPRINT(f,"crossingphase2: yes%s\n",""); break;
+	default: INDPRINT(f,"crossingphase2: no%s\n",""); break;
+	}
+	switch (local_unwind) {
+	case 1:  INDPRINT(f,"crossingoptimization: yes%s\n",""); break;
+	default: INDPRINT(f,"crossingoptimization: no%s\n",""); break;
+	}
+	switch (crossing_heuristics) {
+	case 0:  INDPRINT(f,"crossingweight: bary%s\n",""); break;
+	case 1:  INDPRINT(f,"crossingweight: median%s\n",""); break;
+	case 2:  INDPRINT(f,"crossingweight: barymedian%s\n",""); break;
+	case 3:  INDPRINT(f,"crossingweight: medianbary%s\n",""); break;
+	}
+	switch (crossing_heuristics) {
+	case CSCF_VIEW:  INDPRINT(f,"view: cfish%s\n","");  break;
+	case PSCF_VIEW:  INDPRINT(f,"view: pfish%s\n","");  break;
+	case FCSCF_VIEW: INDPRINT(f,"view: fcfish%s\n",""); break;
+	case FPSCF_VIEW: INDPRINT(f,"view: fpfish%s\n",""); break;
+	}
+	switch (G_arrowmode) {
+	case AMFIXED:  INDPRINT(f,"arrow_mode: fixed%s\n",""); break;
+	case AMFREE:   INDPRINT(f,"arrow_mode: free%s\n",""); break;
 	}
 	switch (layout_flag) {
 	case 1: INDPRINT(f,"layoutalgorithm: maxdepth%s\n","");     break;
@@ -302,6 +384,7 @@ FILE *f;
 	case 9: INDPRINT(f,"layoutalgorithm: maxoutdegree%s\n",""); break;
 	case 10:INDPRINT(f,"layoutalgorithm: mindegree%s\n","");    break;
 	case 11:INDPRINT(f,"layoutalgorithm: maxdegree%s\n","");    break;
+	case 12:INDPRINT(f,"layoutalgorithm: dfs%s\n","");    	    break;
 	}
 	if (info_name_available) {
 		for (k=0; k<3; k++) {
@@ -316,6 +399,11 @@ FILE *f;
 			INDPRINT(f,"hidden: %d\n", k+1);
 		}
 	}
+	for (k=0; k<cmap_size-1; k++) {
+			INDPRINT(f,"colorentry %d : ", k);
+			FPRINTF( f," %d %d %d\n", 
+				redmap[k], greenmap[k], bluemap[k]);
+	}
 	if (class_name_available) {
 		for (k=0; k<max_nr_classes; k++) {
 			if (class_names[k]) {
@@ -331,9 +419,13 @@ FILE *f;
 /*   Print a node 			         		      */
 /*--------------------------------------------------------------------*/
 
+#ifdef ANSI_C
+static void print_node(FILE *f, GNODE v)
+#else
 static void print_node(f,v)
 FILE *f;
 GNODE v;
+#endif
 {
 	INDPRINT(f,"node: {%s\n","");
 	actindent++;
@@ -351,8 +443,8 @@ GNODE v;
 	INDPRINT(f,"width: %d\n",        NWIDTH(v));
 	INDPRINT(f,"height: %d\n",       NHEIGHT(v));
 	INDPRINT(f,"borderwidth: %d\n",  NBORDERW(v));
-	INDPRINT(f,"loc: { x: %d \n",    NX(v));
-	INDPRINT(f,"       y: %d }\n",   NY(v));
+	INDPRINT(f,"loc: { x: %ld \n",    NX(v));
+	INDPRINT(f,"       y: %ld }\n",   NY(v));
 	INDPRINT(f,"shrink:  %d\n",  NSHRINK(v));
 	INDPRINT(f,"stretch: %d\n",  NSTRETCH(v));
 	INDPRINT(f,"textmode: %s\n", textmode_text(NTEXTMODE(v)));
@@ -364,10 +456,14 @@ GNODE v;
 }
 
 
+#ifdef ANSI_C
+static void tbend_node(FILE *f, GEDGE e, int k)
+#else
 static void tbend_node(f,e,k)
 FILE *f;
 GEDGE e;
 int k;
+#endif
 {
 	INDPRINT(f,"node: {%s\n","");
 	actindent++;
@@ -378,17 +474,21 @@ int k;
 	INDPRINT(f,"width: %d\n", 0);
 	INDPRINT(f,"height:%d\n", 0);
 	INDPRINT(f,"borderwidth: %d\n", 0);
-	INDPRINT(f,"loc: { x: %d \n",    ETBENDX(e)+(k-1)*5);
-	INDPRINT(f,"       y: %d }\n",   ETBENDY(e) );
+	INDPRINT(f,"loc: { x: %ld \n",    ETBENDX(e)+(k-1)*5);
+	INDPRINT(f,"       y: %ld }\n",   ETBENDY(e) );
 	actindent--;
 	INDPRINT(f,"}%s\n","");
 }
 
 
+#ifdef ANSI_C
+static void bbend_node(FILE *f, GEDGE e, int k)
+#else
 static void bbend_node(f,e,k)
 FILE *f;
 GEDGE e;
 int k;
+#endif
 {
 	INDPRINT(f,"node: {%s\n","");
 	actindent++;
@@ -399,8 +499,8 @@ int k;
 	INDPRINT(f,"width: %d\n", 0);
 	INDPRINT(f,"height:%d\n", 0);
 	INDPRINT(f,"borderwidth: %d\n", 0);
-	INDPRINT(f,"loc: { x: %d \n",    EBBENDX(e)+(k-1)*5);
-	INDPRINT(f,"       y: %d }\n",   EBBENDY(e) );
+	INDPRINT(f,"loc: { x: %ld \n",    EBBENDX(e)+(k-1)*5);
+	INDPRINT(f,"       y: %ld }\n",   EBBENDY(e) );
 	actindent--;
 	INDPRINT(f,"}%s\n","");
 }
@@ -412,11 +512,15 @@ int k;
 static char lastnode[128];
 static char nextnode[128];
 
+#ifdef ANSI_C
+static void print_edge(FILE *f, GEDGE e)
+#else
 static void print_edge(f,e)
 FILE *f;
 GEDGE e;
+#endif
 {
-	GNODE vs,ve,vh;
+	GNODE vs,ve;
 	char  topbend;
 	char  botbend;
 	int   x1,x2,x3,x4,y1,y2,y3,y4,s;
@@ -457,11 +561,6 @@ GEDGE e;
 
 	if (topbend) tbend_node(f,e,1);
 	if (botbend) bbend_node(f,e,1);
-	if (EART(e)=='D') {
-		if (topbend) tbend_node(f,e,2);
-		if (botbend) bbend_node(f,e,2);
-	}
-	if (EART(e)=='R') { vh=vs; vs=ve; ve=vh; }	
 
 	anchored = 0;
 	if (!NTITLE(vs)||!NTITLE(vs)[0]) {
@@ -470,146 +569,8 @@ GEDGE e;
 	}
 	else 	lasttitle = NTITLE(vs);
 
-	if (topbend && (EART(e)!='R')) {
-		SPRINTF(nextnode,"_inttbend1_%ld",(long)e);
-		nexttitle = nextnode;
-		INDPRINT(f,"edge: {%s\n","");
-		actindent++;
-	        INDPRINT(f,"sourcename: \"%s\"\n",lasttitle);
-		INDPRINT(f,"targetname: \"%s\"\n",nexttitle);
-		if ((!anchored) && (EANCHOR(e)<0))
-			{ INDPRINT(f,"anchor: %d\n",    -EANCHOR(e)); }
-		anchored = 1;
-		INDPRINT(f,"color: %s\n",     color_text(ECOLOR(e)));
-		INDPRINT(f,"linestyle: %s\n", linestyle_text(ELSTYLE(e)));
-		INDPRINT(f,"thickness: %d\n",     ETHICKNESS(e));
-		INDPRINT(f,"class: %d\n",         ECLASS(e));
-		INDPRINT(f,"horizontal_order: %d\n",EHORDER(e));
-		INDPRINT(f,"arrowsize: %d\n",     0);
-		actindent--;
-		INDPRINT(f,"}%s\n","");
-		strcpy(lastnode,nextnode);
-		lasttitle = lastnode;
-	}
-	if (botbend && (EART(e)=='R')) {
-		SPRINTF(nextnode,"_intbbend1_%ld",(long)e);
-		nexttitle = nextnode;
-		INDPRINT(f,"edge: {%s\n","");
-		actindent++;
-	        INDPRINT(f,"sourcename: \"%s\"\n",lasttitle);
-		INDPRINT(f,"targetname: \"%s\"\n",nexttitle);
-		if ((!anchored) && (EANCHOR(e)<0))
-			{ INDPRINT(f,"anchor: %d\n",    -EANCHOR(e)); }
-		anchored = 1;
-		INDPRINT(f,"color: %s\n",     color_text(ECOLOR(e)));
-		INDPRINT(f,"linestyle: %s\n", linestyle_text(ELSTYLE(e)));
-		INDPRINT(f,"thickness: %d\n",     ETHICKNESS(e));
-		INDPRINT(f,"class: %d\n",         ECLASS(e));
-		INDPRINT(f,"horizontal_order: %d\n",EHORDER(e));
-		INDPRINT(f,"arrowsize: %d\n",     0);
-		actindent--;
-		INDPRINT(f,"}%s\n","");
-		strcpy(lastnode,nextnode);
-		lasttitle = lastnode;
-	}
-	if (botbend && (EART(e)!='R')) {
-		SPRINTF(nextnode,"_intbbend1_%ld",(long)e);
-		nexttitle = nextnode;
-		INDPRINT(f,"edge: {%s\n","");
-		actindent++;
-	        INDPRINT(f,"sourcename: \"%s\"\n",lasttitle);
-		INDPRINT(f,"targetname: \"%s\"\n",nexttitle);
-		if ((!anchored) && (EANCHOR(e)<0))
-			{ INDPRINT(f,"anchor: %d\n",    -EANCHOR(e)); }
-		anchored = 1;
-		INDPRINT(f,"color: %s\n",     color_text(ECOLOR(e)));
-		INDPRINT(f,"linestyle: %s\n", linestyle_text(ELSTYLE(e)));
-		INDPRINT(f,"thickness: %d\n",     ETHICKNESS(e));
-		INDPRINT(f,"class: %d\n",         ECLASS(e));
-		INDPRINT(f,"horizontal_order: %d\n",EHORDER(e));
-		INDPRINT(f,"arrowsize: %d\n",     0);
-		actindent--;
-		INDPRINT(f,"}%s\n","");
-		strcpy(lastnode,nextnode);
-		lasttitle = lastnode;
-	}
-	if (topbend && (EART(e)=='R')) {
-		SPRINTF(nextnode,"_inttbend1_%ld",(long)e);
-		nexttitle = nextnode;
-		INDPRINT(f,"edge: {%s\n","");
-		actindent++;
-	        INDPRINT(f,"sourcename: \"%s\"\n",lasttitle);
-		INDPRINT(f,"targetname: \"%s\"\n",nexttitle);
-		if ((!anchored) && (EANCHOR(e)<0))
-			{ INDPRINT(f,"anchor: %d\n",    -EANCHOR(e)); }
-		anchored = 1;
-		INDPRINT(f,"color: %s\n",     color_text(ECOLOR(e)));
-		INDPRINT(f,"linestyle: %s\n", linestyle_text(ELSTYLE(e)));
-		INDPRINT(f,"thickness: %d\n",     ETHICKNESS(e));
-		INDPRINT(f,"class: %d\n",         ECLASS(e));
-		INDPRINT(f,"horizontal_order: %d\n",EHORDER(e));
-		INDPRINT(f,"arrowsize: %d\n",     0);
-		actindent--;
-		INDPRINT(f,"}%s\n","");
-		strcpy(lastnode,nextnode);
-		lasttitle = lastnode;
-	}
-	if (!NTITLE(ve)||!NTITLE(ve)[0]) {
-		SPRINTF(nextnode,"_internal_%ld",(long)ve);
-		nexttitle = nextnode;
-	}
-	else 	nexttitle = NTITLE(ve);
-	INDPRINT(f,"edge: {%s\n","");
-	actindent++;
-	INDPRINT(f,"sourcename: \"%s\"\n",lasttitle);
-	INDPRINT(f,"targetname: \"%s\"\n",nexttitle);
-	if ((!anchored) && (EANCHOR(e)<0))
-		{ INDPRINT(f,"anchor: %d\n",    -EANCHOR(e)); }
-	INDPRINT(f,"color: %s\n",     color_text(ECOLOR(e)));
-	INDPRINT(f,"linestyle: %s\n", linestyle_text(ELSTYLE(e)));
-	INDPRINT(f,"thickness: %d\n",     ETHICKNESS(e));
-	INDPRINT(f,"class: %d\n",         ECLASS(e));
-	INDPRINT(f,"horizontal_order: %d\n",EHORDER(e));
-	if (EORI(e)==NO_ORI) {
-		INDPRINT(f,"arrowsize: %d\n",     0); 
-	}
-	else	INDPRINT(f,"arrowsize: %d\n",     EARROWSIZE(e));
-	actindent--;
-	INDPRINT(f,"}%s\n","");
-
-	if (EART(e)!='D') return;
-	vh=vs; vs=ve; ve=vh; 
-
-	anchored = 0;
-	if (!NTITLE(vs)||!NTITLE(vs)[0]) {
-		SPRINTF(lastnode,"_internal_%ld",(long)vs);
-		lasttitle = lastnode;
-	}
-	else 	lasttitle = NTITLE(vs);
-
-	if (botbend) {
-		SPRINTF(nextnode,"_intbbend2_%ld",(long)e);
-		nexttitle = nextnode;
-		INDPRINT(f,"edge: {%s\n","");
-		actindent++;
-	        INDPRINT(f,"sourcename: \"%s\"\n",lasttitle);
-		INDPRINT(f,"targetname: \"%s\"\n",nexttitle);
-		if ((!anchored) && (EANCHOR(e)<0))
-			{ INDPRINT(f,"anchor: %d\n",    -EANCHOR(e)); }
-		anchored = 1;
-		INDPRINT(f,"color: %s\n",     color_text(ECOLOR(e)));
-		INDPRINT(f,"linestyle: %s\n", linestyle_text(ELSTYLE(e)));
-		INDPRINT(f,"thickness: %d\n",     ETHICKNESS(e));
-		INDPRINT(f,"class: %d\n",         ECLASS(e));
-		INDPRINT(f,"horizontal_order: %d\n",EHORDER(e));
-		INDPRINT(f,"arrowsize: %d\n",     0);
-		actindent--;
-		INDPRINT(f,"}%s\n","");
-		strcpy(lastnode,nextnode);
-		lasttitle = lastnode;
-	}
 	if (topbend) {
-		SPRINTF(nextnode,"_inttbend2_%ld",(long)e);
+		SPRINTF(nextnode,"_inttbend1_%ld",(long)e);
 		nexttitle = nextnode;
 		INDPRINT(f,"edge: {%s\n","");
 		actindent++;
@@ -619,11 +580,63 @@ GEDGE e;
 			{ INDPRINT(f,"anchor: %d\n",    -EANCHOR(e)); }
 		anchored = 1;
 		INDPRINT(f,"color: %s\n",     color_text(ECOLOR(e)));
+		INDPRINT(f,"textcolor: %s\n", color_text(ELABELCOL(e)));
+		INDPRINT(f,"arrowcolor: %s\n", color_text(EARROWCOL(e)));
+		INDPRINT(f,"backarrowcolor: %s\n", color_text(EARROWBCOL(e)));
 		INDPRINT(f,"linestyle: %s\n", linestyle_text(ELSTYLE(e)));
 		INDPRINT(f,"thickness: %d\n",     ETHICKNESS(e));
 		INDPRINT(f,"class: %d\n",         ECLASS(e));
 		INDPRINT(f,"horizontal_order: %d\n",EHORDER(e));
 		INDPRINT(f,"arrowsize: %d\n",     0);
+		INDPRINT(f,"backarrowsize: %d\n", EARROWBSIZE(e));
+		INDPRINT(f,"arrowstyle: none%s\n", "");
+		switch (EARROWSIZE(e)) {
+		case ASNONESPEC: 
+		case ASNONE: INDPRINT(f,"backarrowstyle:none%s\n", ""); break;
+		case ASSOLID:INDPRINT(f,"backarrowstyle:solid%s\n",""); break;
+		case ASLINE: INDPRINT(f,"backarrowstyle:line%s\n", ""); break;
+		}
+		actindent--;
+		INDPRINT(f,"}%s\n","");
+		strcpy(lastnode,nextnode);
+		lasttitle = lastnode;
+	}
+	if (botbend) {
+		SPRINTF(nextnode,"_intbbend1_%ld",(long)e);
+		nexttitle = nextnode;
+		INDPRINT(f,"edge: {%s\n","");
+		actindent++;
+	        INDPRINT(f,"sourcename: \"%s\"\n",lasttitle);
+		INDPRINT(f,"targetname: \"%s\"\n",nexttitle);
+		if ((!anchored) && (EANCHOR(e)<0))
+			{ INDPRINT(f,"anchor: %d\n",    -EANCHOR(e)); }
+		anchored = 1;
+		INDPRINT(f,"color: %s\n",     color_text(ECOLOR(e)));
+		INDPRINT(f,"textcolor: %s\n", color_text(ELABELCOL(e)));
+		INDPRINT(f,"arrowcolor: %s\n", color_text(EARROWCOL(e)));
+		INDPRINT(f,"backarrowcolor: %s\n", color_text(EARROWBCOL(e)));
+		INDPRINT(f,"linestyle: %s\n", linestyle_text(ELSTYLE(e)));
+		INDPRINT(f,"thickness: %d\n",     ETHICKNESS(e));
+		INDPRINT(f,"class: %d\n",         ECLASS(e));
+		INDPRINT(f,"horizontal_order: %d\n",EHORDER(e));
+		INDPRINT(f,"arrowsize: %d\n",     0);
+		if (!topbend) {
+			INDPRINT(f,"backarrowsize: %d\n", EARROWBSIZE(e));
+		}
+		else	INDPRINT(f,"backarrowsize: %d\n", 0);
+		INDPRINT(f,"arrowstyle: none%s\n", "");
+		if (!topbend) {
+			switch (EARROWSIZE(e)) {
+			case ASNONESPEC: 
+			case ASNONE: INDPRINT(f,"backarrowstyle:none%s\n", "");
+				     break;
+			case ASSOLID:INDPRINT(f,"backarrowstyle:solid%s\n","");
+				     break;
+			case ASLINE: INDPRINT(f,"backarrowstyle:line%s\n", "");
+				     break;
+			}
+		}
+		else INDPRINT(f,"backarrowstyle: none%s\n", "");
 		actindent--;
 		INDPRINT(f,"}%s\n","");
 		strcpy(lastnode,nextnode);
@@ -641,14 +654,34 @@ GEDGE e;
 	if ((!anchored) && (EANCHOR(e)<0))
 		{ INDPRINT(f,"anchor: %d\n",    -EANCHOR(e)); }
 	INDPRINT(f,"color: %s\n",     color_text(ECOLOR(e)));
+	INDPRINT(f,"textcolor: %s\n", color_text(ELABELCOL(e)));
+	INDPRINT(f,"arrowcolor: %s\n", color_text(EARROWCOL(e)));
+	INDPRINT(f,"backarrowcolor: %s\n", color_text(EARROWBCOL(e)));
+	INDPRINT(f,"label: \"%s\"\n", ELABEL(e));
 	INDPRINT(f,"linestyle: %s\n", linestyle_text(ELSTYLE(e)));
 	INDPRINT(f,"thickness: %d\n",     ETHICKNESS(e));
 	INDPRINT(f,"class: %d\n",         ECLASS(e));
 	INDPRINT(f,"horizontal_order: %d\n",EHORDER(e));
-	if (EORI(e)==NO_ORI) {
-		INDPRINT(f,"arrowsize: %d\n",     0); 
+	INDPRINT(f,"arrowsize: %d\n",     EARROWSIZE(e));
+	if (!topbend && !botbend) {
+		INDPRINT(f,"backarrowsize: %d\n", EARROWBSIZE(e));
 	}
-	else	INDPRINT(f,"arrowsize: %d\n",     EARROWSIZE(e));
+	else	INDPRINT(f,"backarrowsize: %d\n", 0);
+	switch (EARROWSIZE(e)) {
+	case ASNONESPEC: 
+	case ASNONE:  INDPRINT(f,"arrowstyle: none%s\n", ""); break;
+	case ASSOLID: INDPRINT(f,"arrowstyle: solid%s\n",""); break;
+	case ASLINE:  INDPRINT(f,"arrowstyle: line%s\n", ""); break;
+	}
+	if (!topbend && !botbend) {
+		switch (EARROWSIZE(e)) {
+		case ASNONESPEC: 
+		case ASNONE:  INDPRINT(f,"backarrowstyle: none%s\n", ""); break;
+		case ASSOLID: INDPRINT(f,"backarrowstyle: solid%s\n",""); break;
+		case ASLINE:  INDPRINT(f,"backarrowstyle: line%s\n", ""); break;
+		}
+	}
+	else INDPRINT(f,"backarrowstyle: none%s\n", "");
 	actindent--;
 	INDPRINT(f,"}%s\n","");
 }
@@ -657,8 +690,14 @@ GEDGE e;
 /*   Convert a color into its name				      */
 /*--------------------------------------------------------------------*/
 
+static char mycolbuf[10];
+
+#ifdef ANSI_C
+char *color_text(int c)
+#else
 char *color_text(c)
 int c;
+#endif
 {
 	switch (c) {
 	case BLACK       : return("black");
@@ -694,7 +733,9 @@ int c;
 	case ORANGE      : return("orange");
 	case ORCHID      : return("orchid");
 	}
-	return("none");
+
+	SPRINTF(mycolbuf,"%d",c);
+	return(mycolbuf);
 }
 
 
@@ -703,8 +744,12 @@ int c;
 /*   Convert a textmode into its name				      */
 /*--------------------------------------------------------------------*/
 
+#ifdef ANSI_C
+char *textmode_text(int t)
+#else
 char *textmode_text(t)
 int t;
+#endif
 {
 	switch (t) {
 	case CENTER: return("center");
@@ -718,8 +763,12 @@ int t;
 /*   Convert a shape into its name				      */
 /*--------------------------------------------------------------------*/
 
+#ifdef ANSI_C
+char *shape_text(int t)
+#else
 char *shape_text(t)
 int t;
+#endif
 {
 	switch (t) {
 	case BOX:      return("box");
@@ -736,13 +785,18 @@ int t;
 /*   Convert a linestyle into its name				      */
 /*--------------------------------------------------------------------*/
 
+#ifdef ANSI_C
+char *linestyle_text(int t)
+#else
 char *linestyle_text(t)
 int t;
+#endif
 {
 	switch (t) {
-	case SOLID:  return("continuous");
-	case DOTTED: return("dotted");
-	case DASHED: return("dashed");
+	case SOLID:     return("continuous");
+	case DOTTED:    return("dotted");
+	case DASHED:    return("dashed");
+	case UNVISIBLE: return("invisible");
 	}
 	return("none");
 }
